//****************************************************************************
//
// chronos_drive.c - Controle l'EvalBot via UART bas sur Remote control of
// the EVALBOT using an eZ430-Chronos.
//
//****************************************************************************
#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
#include <string.h>
#include "driverlib/uart.h"
#include "I2Clib.h"
#include "driverlib/timer.h"
#include "driverlib/adc.h"

//*****************************************************************************
//
// Defines
//
//*****************************************************************************
#define CLEAR 1
#define HideCurs    "\033[?25l"
#define ClearScreen "\033[H\033[2J"
// This example code is in the public domain.
// 0xE0 = 112 dec sur 7 bits de PF
// 0XE2 = 113
// 0XE4 = 114

#define srf02_g 112
#define srf02_g 112
//*****************************************************************************
//
// Main application entry point.
//
//*****************************************************************************
int main (void)
{
   unsigned long int i,j;
   unsigned long u;
   //
   // Set the system clock to run at 50MHz from the PLL
   //
   SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN | SYSCTL_XTAL_16MHZ);

   //
   // Enable UART0.
   //
   SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
   SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
   GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
   //
   // Initialize the UART standard I/O.
   //
   UARTStdioInit(0);
	UARTConfigSetExpClk( UART0_BASE, SysCtlClockGet(), 9600,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                        UART_CONFIG_PAR_NONE) );
   //                    
   // pour activer le convertisseur +5v  PBA6 = 1
   //
   GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, (GPIO_PIN_6) );
   GPIOPadConfigSet( GPIO_PORTA_BASE, GPIO_PIN_6, GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );
   GPIOPinWrite( GPIO_PORTA_BASE,(GPIO_PIN_6), 0x40);
   //
   // Print a nice welcoming banner on the serial output.
   //
	UARTprintf(ClearScreen);  // efface la console 
	UARTprintf(HideCurs);	  // efface le curseur
   UARTprintf("EVALBOT test ADC simple\n");
   UARTprintf("-----------------------\n");
   //
   // Initialize the LED display.
   //
   Display96x16x1Init(true);
   Display96x16x1Clear();
	Display96x16x1StringDrawCentered("TesT ADC Simple", 0, CLEAR);
   
   
   
   // Enable the ADC hardware
    SysCtlPeripheralEnable(SYSCTL_PERIPH_ADC0);

    // Configure the pin as analog input
    SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOB ); // PB
    GPIOPinTypeADC( GPIO_PORTB_BASE, GPIO_PIN_4 ); // PB4

    // pour 1 ADC
    // use Sample Sequencer 0 since it is the only one able to handle more than four ADCs
    ADCSequenceDisable(ADC0_BASE, 0);
    // configure Sequencer to trigger from processor with priority 0
    ADCSequenceConfigure(ADC0_BASE, 0, ADC_TRIGGER_PROCESSOR, 0);
    // configure the steps of the Sequencer
    // Enable the first sample sequencer to capture the value of channel 0 when
    // the processor trigger occurs.
    ADCSequenceStepConfigure(ADC0_BASE, 0, 0, ADC_CTL_IE | ADC_CTL_END | ADC_CTL_CH10);   // PB4
    ADCSequenceEnable(ADC0_BASE, 0);

   for (;;) {
      //
      // Trigger the sample sequence.
      //
      ADCProcessorTrigger( ADC0_BASE, 0 );
      //
      // Wait until the sample sequence has completed.
      //
      while(!ADCIntStatus( ADC0_BASE, 0, false ));
      //
      // Read the value from the ADC.
      //
      ADCSequenceDataGet( ADC0_BASE, 0, &u );
      UARTprintf("\33[4;0H adc Port B 4 = %X \n", u);
      Display96x16x1ClearLine(1);
      Display96x16x1StringDrawCenteredPrintf( 0,0, "%u",u );
   }
}
