//****************************************************************************
//
// chronos_drive.c - Controle l'EvalBot via UART bas sur Remote control of
// the EVALBOT using an eZ430-Chronos.
//
//****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
#include "sounds.h"
#include <string.h>
#include "driverlib/uart.h"
#include "driverlib/timer.h"

//*****************************************************************************
//
// For debug purposes, define this label to keep the motors disabled at all
// times.
//
//*****************************************************************************
//#define KEEP_MOTORS_DISABLED

//*****************************************************************************
//
// Defines
//
//*****************************************************************************
#define CLEAR 1
#define HideCurs    "\033[?25l"
#define ClearScreen "\033[H\033[2J"

static unsigned int p1,p2,p;
static unsigned char rising;

void UARTelipse()
{
  	static unsigned char j = 0;
	static char elipse[] = "|/-\\|/-\\";

	UARTprintf("\33[4;5H");
	UARTwrite(&elipse[j],1);
	j++; if (j >= 8) j = 0;
}

//*****************************************************************************
//
// Main application entry point.
//
//*****************************************************************************
void PortCIntHandler()
{
	GPIOPinIntClear( GPIO_PORTC_BASE, (GPIO_PIN_5) );
	if (rising == 0) {
		TimerLoadSet(TIMER0_BASE, TIMER_A, 0xffffff);
		TimerEnable(TIMER0_BASE, TIMER_A);
		rising = 1;
	} else {		
		p = TimerValueGet(TIMER0_BASE, TIMER_A);
		TimerDisable(TIMER0_BASE, TIMER_A);
		rising = 0;
	}
}

void PulseCaptureIntHandler()
{
    // Clear Timer3_B interrupt flag
    TimerIntClear(TIMER0_BASE,TIMER_CAPB_EVENT);
//	p1 = p2;
//  	p2 = TimerValueGet(TIMER0_BASE, TIMER_B);
	
	if (rising == 0) {
		p1 = TimerValueGet(TIMER0_BASE, TIMER_B);
		rising = 1;
	} else {
		p2 = TimerValueGet(TIMER0_BASE, TIMER_B);
		if (p1 > p2	)
			p = p1 - p2;
		else
			p = p1 + ( 0xffff - p2);
		rising = 0;
		TimerDisable(TIMER0_BASE, TIMER_B);		
	}
}

int main (void)
{
	unsigned long int i;
	 unsigned int d;

    //
    // Set the system clock to run at 50MHz from the PLL
    //
    SysCtlClockSet(SYSCTL_SYSDIV_50 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_16MHZ);
    //
    // Enable UART0.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);

    //
    // Initialize the UART 0 port serie sur USB
    //  

	UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), 115200,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));

    //
    // Initialize and Enable UART1 port serie sur pins
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART1);

	GPIOPinConfigure(GPIO_PB4_U1RX);
    GPIOPinConfigure(GPIO_PB5_U1TX);

    GPIOPinTypeUART(GPIO_PORTB_BASE, GPIO_PIN_4 | GPIO_PIN_5);

	UARTConfigSetExpClk(UART1_BASE, SysCtlClockGet(), 9600,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));
 
    //
    // Initialize the UART standard I/O.
    //
    UARTStdioInit(1); // 0 pour rediriger es STDIO sur le UART O (port serie usb)
 				      // 1 pour rediriger es STDIO sur le UART 1 (port serie Xbee)
    //
    // Print a nice welcoming banner on the serial output.
    //

	UARTprintf(ClearScreen);  // efface la console 
    UARTprintf("EVALBOT GPIO\n");
    UARTprintf("------------\n");
	UARTprintf(HideCurs);	  // efface le curseur
    //
    // Initialize the LED display.
    //
    Display96x16x1Init(true);
    Display96x16x1Clear();
	Display96x16x1StringDrawCentered("TEST SRF05", 0, CLEAR);
	//Display96x16x1StringDrawCentered("TEST GPIO", 1, CLEAR);
    //
    // Initialize the board LEDs.
    //
    LEDsInit();

	//
	// Initialize the GPIO pin configuration.
	//
	// Set pins 1 as output, SW controlled.
	//
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC); 

	GPIOPinTypeGPIOOutput( GPIO_PORTC_BASE, (GPIO_PIN_4) ); // PC4 en sortie
	GPIOPinTypeGPIOInput(  GPIO_PORTC_BASE, (GPIO_PIN_5) ); //PC5 en entre

	// PC4 en sortie standard + 2 mA 
	GPIOPadConfigSet( GPIO_PORTC_BASE, (GPIO_PIN_4), GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );
 
	// SRF05 l'echo mesure entre 0,100ms et 25ms  = 30ms si aucun objet detect
	// La broche d'echo  du SRF05 est reli a PC5
	// PC5 en mode CCP1 possible sur TimerB tu Timer 0
	//
	SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER0);

	GPIOPinConfigure(GPIO_PC5_CCP1);        
    GPIOPinTypeTimer(GPIO_PORTC_BASE, GPIO_PIN_5);

	TimerConfigure(TIMER0_BASE, ( TIMER_CFG_16_BIT_PAIR | TIMER_CFG_B_CAP_TIME ));

	TimerControlEvent( TIMER0_BASE, TIMER_B, TIMER_EVENT_BOTH_EDGES );
	TimerIntEnable(TIMER0_BASE, TIMER_CAPB_EVENT);
	TimerIntRegister(TIMER0_BASE, TIMER_B, PulseCaptureIntHandler);

	TimerEnable(TIMER0_BASE, TIMER_B);

//	TimerPrescaleSet( TIMER0_BASE, TIMER_B, 100);  // on divise l'horloge systeme 

	IntMasterEnable();

	for (;;) {
//	    UARTelipse();
//		GPIOPinWrite( GPIO_PORTC_BASE,(GPIO_PIN_4), 0x00);
		// Pulse 15us vers SRF05
		rising = 0;

		GPIOPinWrite( GPIO_PORTC_BASE,(GPIO_PIN_4), 0xff);
		for (i=0;i<300;i++);
//	    UARTelipse();
		GPIOPinWrite( GPIO_PORTC_BASE,(GPIO_PIN_4), 0x00);
		TimerLoadSet(TIMER0_BASE, TIMER_B, 0xffff);
		TimerEnable(TIMER0_BASE, TIMER_B);
		for (i=0;i<500000;i++);
		//UARTprintf("\33[4;1H          ");
		UARTprintf("%d\n", p );
		Display96x16x1StringDrawCenteredPrintf(1,1,"   %d\n", p );
	}				
}