//****************************************************************************
//
// EvalBot : exmple d'utilisation de
// Oled
// UART0 ou UART1
// Sonar Ultra-son SRF05 avec GPIO (Irq) + TImer
//
//****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
#include "sounds.h"
#include <string.h>
#include "driverlib/uart.h"
#include "driverlib/timer.h"

//*****************************************************************************
//
// Defines
//
//*****************************************************************************
#define CLEAR 1
#define HideCurs    "\033[?25l"
#define ClearScreen "\033[H\033[2J"

static unsigned int p;
static unsigned char rising;

void UARTelipse()
{
  	static unsigned char j = 0;
	static char elipse[] = "|/-\\|/-\\";

	UARTprintf("\33[4;5H");
	UARTwrite(&elipse[j],1);
	j++; if (j >= 8) j = 0;
}

static void EvalBotStop()
{
    //
    // Stop the motors.
    //
    MotorStop(LEFT_SIDE);
    MotorStop(RIGHT_SIDE);

}
static void
EvalBotRun()
{	
	MotorRun(LEFT_SIDE);
	MotorRun(RIGHT_SIDE);
}

void BumpDetection()
{
	tBoolean l, r;
	unsigned long int c;

	l = !BumpSensorGetStatus(LEFT_SIDE);
	r = !BumpSensorGetStatus(RIGHT_SIDE);

	if( l || r)	{
		EvalBotStop();
		if (l && r ) {
			MotorDir(LEFT_SIDE,  REVERSE);
			MotorDir(RIGHT_SIDE, REVERSE);
		}else{
			if (r) {
				MotorDir(LEFT_SIDE,  REVERSE);
				MotorDir(RIGHT_SIDE, FORWARD);
			}else{
				MotorDir(LEFT_SIDE,  FORWARD);
				MotorDir(RIGHT_SIDE, REVERSE);
			}
		}
		EvalBotRun();
		Display96x16x1StringDrawCentered("!! COLLISION !!", 1, CLEAR);
		UARTprintf("\33[6;0H");
		UARTprintf("Collision          ");		
		for(c=0;c<13000000;c++);
	}
}

//*****************************************************************************
// Ecriture LCD
// RS sur PC4
// R/W sur PH2
// E sur PH3
// DB7..DB4 sur PH7..PH4
//*****************************************************************************
void write_4b_LCD(unsigned char v)
{
   static unsigned char i;

   // On selectionne le Registre d'Instruction : RS <= 0
   GPIOPinWrite( GPIO_PORTC_BASE, GPIO_PIN_4, 0x00); 

   // Cycle d'ecritue des 4 bits de poids forts dans le registre d'Instruction du lcd  
   // R/W <= 0 + E <=0
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, ((v<<4) & 0xf0));
   // attendre Tsp1 = 60ns
   //for (i=0;i<6;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<2;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<1;i++);
}

//**************************

void writeIR_LCD( unsigned char v)
{
   static unsigned int i;

   // Cycle d'ecritue des 4 bits de poids forts dans le registre d'Instruction du lcd  
   // R/W <= 0 + E <=0
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, (v & 0xf0));
   // attendre Tsp1 = 60ns
   for (i=0;i<1;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<4;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<5;i++);

   // On selectionne le Registre d'Instruction : RS <= 0
   GPIOPinWrite( GPIO_PORTC_BASE, GPIO_PIN_4, 0x00); 

   // write lower 4bits of Instruction Register 
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, ((v << 4) & 0xf0 ));
   // attendre Tsp1 = 60ns
   for (i=0;i<1;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<4;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<5;i++);
}

//**************************
void writeDR_LCD( unsigned char v)
{
   static unsigned int i;

   // Cycle d'ecritue des 4 bits de poids forts dans le registre d'Instruction du lcd  
   // R/W <= 0 + E <=0
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, (v & 0xf0));
   // attendre Tsp1 = 60ns
   for (i=0;i<2;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<5;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<6;i++);

   // On selectionne le Registre d'Instruction : RS <= 1
   GPIOPinWrite( GPIO_PORTC_BASE, GPIO_PIN_4, 0x10); 

   // write lower 4bits of Instruction Register 
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, ((v << 4) & 0xf0 ));
   // attendre Tsp1 = 60ns
   for (i=0;i<2;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<5;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<6;i++);

}

void init_LCD()
{
   unsigned long i;
   write_4b_LCD(0x3);
   for (i=0;i<80000;i++); // 5ms
   write_4b_LCD(0x3);
   for (i=0;i<2000;i++); // 200 us
   write_4b_LCD(0x3);
   for (i=0;i<2000;i++);

   write_4b_LCD(0x2);     // interface  4bits
   for (i=0;i<2000;i++); // 2000 us

   writeIR_LCD(0x28); // 2 lignes + matrice de 8x5 points
   for (i=0;i<2000;i++); // 200 us

   writeIR_LCD(0x0f);    // Display OFF
   for (i=0;i<2000;i++); // 200 us

   writeIR_LCD(0x14);  // Display clear
   for (i=0;i<2000;i++); // 200 us
}
void puts_LCD( char *s)
{
   static unsigned long i;
   while (*s != 0 )  {
      writeDR_LCD( *s );
      s++;
      for(i=0;i<10000;i++);
   }
}
//*****************************************************************************
// procedure de traitement de L'it Echo du module SRF05
//*****************************************************************************
static void PortBIntHandler()
{
	GPIOPinIntClear( GPIO_PORTB_BASE, (GPIO_PIN_2) );  // raz 
   
	if (rising == 0) {  // front montant on lance le timer
		TimerLoadSet(TIMER0_BASE, TIMER_A, 0xffffff);
		TimerEnable(TIMER0_BASE, TIMER_A);
		rising = 1; // memo du front montant
	} else {	// front descendant lecture du timer et arret
		p = TimerValueGet(TIMER0_BASE, TIMER_A);
		TimerDisable(TIMER0_BASE, TIMER_A);
		rising = 0; // memo du front descendant 
	}
}
//*****************************************************************************
// MAIN  MAIN  MAIN  MAIN
//*****************************************************************************

int main (void)
{
	static unsigned long int i,j;
	unsigned char c, v;

   //
   // Set the system clock to run at 50MHz from the PLL
   //
   SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_16MHZ);
   
   //
   // Init UART0 port serie sur usb.
   //
   SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
   SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
   GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
  	UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), 115200,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));

   //
   // InitUART1 port serie sur GPIO vers le Xbee (ou Bluetooth)
   //
   SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
   SysCtlPeripheralEnable(SYSCTL_PERIPH_UART1);
   GPIOPinTypeUART(GPIO_PORTB_BASE, GPIO_PIN_4 | GPIO_PIN_5);
   GPIOPinConfigure(GPIO_PB4_U1RX);
   GPIOPinConfigure(GPIO_PB5_U1TX);
	UARTConfigSetExpClk(UART1_BASE, SysCtlClockGet(), 115200,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));
 
   //
   // Init l'UART des standard I/O.
   //
   UARTStdioInit(1); // pour rediriger les STDIO sur le UART 1 (port serie Xbee)
 
   //
   // Affiche un message sur la sortie serie standard
   //
	UARTprintf( ClearScreen );  // efface la console 
   UARTprintf( "EVALBOT INF201 SRF05 + LCD\n" );
   UARTprintf( "--------------------------\n" );
	UARTprintf( HideCurs );	  // efface le curseur

   //
   // Init du oLED EvalBot.
   //
   Display96x16x1Init(true);
   Display96x16x1Clear();
	Display96x16x1StringDrawCentered("Test SRF05", 0, CLEAR);
   //
   // Init des 2 LEDs a l'avant de l'EvalBot.
   //
   LEDsInit();

   /////////////////////////////////////////////
   // INIT gpio pour l'ecran LCD INF201
   // LCD sur le Por H + PortC
   //    DB7..DB4 <=> PH7..PH4
   //    E   <= PH3
   //    R/W <= PH2
   //    RS  <= PC4
  
   SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);
   SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOH);

	GPIOPinTypeGPIOOutput( GPIO_PORTH_BASE, 
                         (GPIO_PIN_7 | GPIO_PIN_6 |
                          GPIO_PIN_5 | GPIO_PIN_4 |
                          GPIO_PIN_3 | GPIO_PIN_2) ); // PH7..PH2 en sortie

	// PH7..PH2 en sortie standard + 2 mA 
	GPIOPadConfigSet( GPIO_PORTH_BASE,
                     (GPIO_PIN_7 | GPIO_PIN_6 |
                      GPIO_PIN_5 | GPIO_PIN_4 |
                      GPIO_PIN_3 | GPIO_PIN_2),
                     GPIO_STRENGTH_2MA,
                     GPIO_PIN_TYPE_STD );

   // PC4 => RS lcd
   GPIOPinTypeGPIOOutput( GPIO_PORTC_BASE, (GPIO_PIN_4) ); 
	GPIOPadConfigSet(      GPIO_PORTC_BASE, (GPIO_PIN_4), GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );
	
	//////////////////////////////////
	// INIT gpio pour le module SRF05
	//  entree Trigger <= PB3
   //  sortie Echo    => PB2
   //
	
	GPIOPinTypeGPIOOutput(  GPIO_PORTB_BASE, (GPIO_PIN_3) ); // PB3 en sortie Trigger SRF05
	GPIOPinTypeGPIOInput(   GPIO_PORTB_BASE, (GPIO_PIN_2) ); // PB2 en entre Echo    SRF05

	// PB3 en sortie standard + 2 mA 
	GPIOPadConfigSet( GPIO_PORTB_BASE, (GPIO_PIN_3), GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );

   // init PB2 Echo pour IT sur fronts montant et descandant
 	GPIOPortIntRegister( GPIO_PORTB_BASE, PortBIntHandler);
	GPIOPinIntEnable(    GPIO_PORTB_BASE, GPIO_PIN_2 );     
	GPIOIntTypeSet(      GPIO_PORTB_BASE, GPIO_PIN_2, GPIO_BOTH_EDGES);
   GPIOPinIntEnable(    GPIO_PORTB_BASE, GPIO_PIN_2);

   // Initialisation du LCD
   init_LCD();
   puts_LCD( "Salut a tous !");

	// SRF05 l'echo mesure entre 0,100ms et 25ms  = 30ms si aucun objet detect
	//
	SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER0);

	TimerConfigure(TIMER0_BASE, ( TIMER_CFG_SPLIT_PAIR | TIMER_CFG_A_ONE_SHOT ));

	TimerPrescaleSet( TIMER0_BASE, TIMER_A, 100);  // on divise l'horloge systeme 
	IntMasterEnable();

   //
   // Initialize the motors.
   //
   MotorsInit();
   v = 30;
   //
   // Initialize the front bump sensors.
   //
   BumpSensorsInit();

   for (;;) {
 
		UARTprintf("\33[6;0H");
		UARTprintf("ARRET                  ");

      do{
         c = UARTgetc();
		   switch (c) {
			   case '+': {
				   if (v < 99) v++;
				   break;
			   }
			   case '-': {
				   if (v > 10) v--;
				   break;
			   }
			   default:
			   break;
		   }
		   UARTprintf("\33[5;0H");
		   UARTprintf("Vitesse a %d%%",v);
		   MotorSpeed(LEFT_SIDE,  v << 8); 
		   MotorSpeed(RIGHT_SIDE, v << 8);
      } while ( c != 27); // attendre la sequence ESC 
	   c = UARTgetc(); // on devrait attendre le code 91
      c = UARTgetc(); 
	   //	 UARTprintf("c = %d\n",c);
		
      switch (c) {
	   	case 65: { // FORWARD
		   	Display96x16x1StringDrawCentered("en AVANT !", 1, CLEAR);
			   UARTprintf("\33[6;0H");
			   UARTprintf("en AVANT !           ");
			   MotorDir(LEFT_SIDE,  FORWARD);
			   MotorDir(RIGHT_SIDE, FORWARD);
			   break;
		   }
		   case 68: { // LEFT
			   Display96x16x1StringDrawCentered("rotation gauche", 1, CLEAR);
			   UARTprintf("\33[6;0H");
				UARTprintf("rotation a gauche    ");
				MotorDir(LEFT_SIDE,  REVERSE);
				MotorDir(RIGHT_SIDE, FORWARD);			     
				break;
			}
			case 67: { // RIGHT
				Display96x16x1StringDrawCentered("rotation droite", 1, CLEAR);
				UARTprintf("\33[6;0H");
				UARTprintf("rotation a droite    ");
				MotorDir(LEFT_SIDE,  FORWARD);
				MotorDir(RIGHT_SIDE, REVERSE);			     
				break;
			}
			case 66: { // REVERSE
				Display96x16x1StringDrawCentered("en ARRIERE !", 1, CLEAR);
				UARTprintf("\33[6;0H");
				UARTprintf("en ARRIERE !       ");
				MotorDir(LEFT_SIDE,  REVERSE);
				MotorDir(RIGHT_SIDE, REVERSE);
				break;
			}
			default: // pour tous autre carac on ne fait rien 
			         // si executra la precedente commande
			break;
		}

		EvalBotRun();

		for(j=0;j<6;j++) {
  			LED_Off(LED_1); LED_On(LED_2);
         // trigger SRF05
         rising = 0;
         GPIOPinWrite( GPIO_PORTB_BASE,(GPIO_PIN_3), 0xff);
         for (i=0;i<300;i++);
         GPIOPinWrite( GPIO_PORTB_BASE,(GPIO_PIN_3), 0x00);
//       for (i=0;i<1000000;i++);
      	UARTprintf("\33[4;5H          ");                    
         UARTprintf("\33[4;5H %d\n", 65535-p );
         Display96x16x1StringDrawCenteredPrintf(0,0,"SRF05 = %u", 65535-p );
			BumpDetection();
			LED_On(LED_1);	LED_Off(LED_2);
			for (i=0;i<200000;i++);
			//BumpDetection();
			//LED_Off(LED_1); LED_On(LED_2);
			//for (i=0;i<400000;i++);
			//LED_Off(LED_2);
		}
      LED_Off(LED_1);		
		EvalBotStop();
	}
}