//******************************
#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
#include "sounds.h"
#include <string.h>
#include "driverlib/uart.h"
#include "driverlib/timer.h"
//*****************************************************************************
// Ecriture LCD
// RS sur PC4
// R/W sur PH2
// E sur PH3
// DB7..DB4 sur PH7..PH4
//*****************************************************************************
void write_4b_LCD(unsigned char v)
{
   static unsigned char i;

   // On selectionne le Registre d'Instruction : RS <= 0
   GPIOPinWrite( GPIO_PORTC_BASE, GPIO_PIN_4, 0x00); 

   // Cycle d'ecritue des 4 bits de poids forts dans le registre d'Instruction du lcd  
   // R/W <= 0 + E <=0
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, ((v<<4) & 0xf0));
   // attendre Tsp1 = 60ns
   //for (i=0;i<6;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<2;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<1;i++);
}

//**************************

void writeIR_LCD( unsigned char v)
{
   static unsigned int i;

   // Cycle d'ecritue des 4 bits de poids forts dans le registre d'Instruction du lcd  
   // R/W <= 0 + E <=0
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, (v & 0xf0));
   // attendre Tsp1 = 60ns
   for (i=0;i<1;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<4;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<5;i++);

   // On selectionne le Registre d'Instruction : RS <= 0
   GPIOPinWrite( GPIO_PORTC_BASE, GPIO_PIN_4, 0x00); 

   // write lower 4bits of Instruction Register 
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, ((v << 4) & 0xf0 ));
   // attendre Tsp1 = 60ns
   for (i=0;i<1;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<4;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<5;i++);
}

//**************************
void writeDR_LCD( unsigned char v)
{
   static unsigned char i;

   // Cycle d'ecritue des 4 bits de poids forts dans le registre d'Instruction du lcd  
   // R/W <= 0 + E <=0
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, (v & 0xf0));
   // attendre Tsp1 = 60ns
   for (i=0;i<2;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<5;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<6;i++);

   // On selectionne le Registre d'Instruction : RS <= 1
   GPIOPinWrite( GPIO_PORTC_BASE, GPIO_PIN_4, 0x10); 

   // write lower 4bits of Instruction Register 
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, ((v << 4) & 0xf0 ));
   // attendre Tsp1 = 60ns
   for (i=0;i<2;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<5;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<6;i++);

}

void init_LCD()
{
   unsigned long i;
   
   write_4b_LCD(0x3);
   for (i=0;i<60000;i++); // 4.8ms
   write_4b_LCD(0x3);
   for (i=0;i<2000;i++); // 160 us
   write_4b_LCD(0x3);
   for (i=0;i<2000;i++); // 160 us

   write_4b_LCD(0x2);     // interface  4bits
   for (i=0;i<2000;i++); // 2000 us

   writeIR_LCD(0x28); // 2 lignes + matrice de 8x5 points
   for (i=0;i<2000;i++); // 160 us

   writeIR_LCD(0x0f);    // Display OFF
   for (i=0;i<2000;i++); // 160 us

   writeIR_LCD(0x14);  // Display clear
   for (i=0;i<2000;i++); // 160 us
	
	 writeIR_LCD(0x0c);  // cusor off + blink off
   for (i=0;i<2000;i++); // 160 us
}
//**************************
void puts_LCD( char *s)
{
   static unsigned long i;
   while (*s != 0 )  {
      writeDR_LCD( *s );
      s++;
      for(i=0;i<50000;i++);
   }
}
