;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; EL 09/2013 - Evalbot (Cortex M3 de Texas Instrument)
; exemple1 LCD (pas de lecture du busy flag)
; le signal R/W\ du lcd toujours en mode WRITE
;
   	
		         AREA    |.text|, CODE, READONLY

SYSCTL         equ   0x400FE000  ; System Control base address
SYSCTL_RCGC2   equ   0x400FE108  ; Run Mode Clock Gating Control Register 2   p292
                                 ; pour enable clock des GPIOs

SYSCTL_PERIPH	EQU	0x400FE108	; SYSCTL_RCGC2_R : pour enable clock (p291 datasheet de lm3s9b92.pdf)
							         	; voir chap.8 "GPIO": page 416 de lm3s9B92.pdf

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GPIO Register MAP page 413-414 du lm3s9b92.pdf
; les adresses :
;
GPIO_PORTA_BASE   EQU	0x40004000  
GPIO_PORTB_BASE   EQU	0x40005000
GPIO_PORTC_BASE   EQU	0x40006000
GPIO_PORTD_BASE   EQU	0x40007000	; GPIO Port D (APB) base: 0x4000.7000 (p416 datasheet de lm3s9B92.pdf)
GPIO_PORTE_BASE   EQU	0x40024000
GPIO_PORTF_BASE	EQU	0x40025000	; GPIO Port F (APB) base: 0x4002.5000 (p416 datasheet de lm3s9B92.pdf)
GPIO_PORTG_BASE   EQU	0x40026000
GPIO_PORTH_BASE   EQU   0x40027000
GPIO_PORTJ_BASE	EQU	0x4003D000  ; GPIO Port J (APB) base: 0x4003.D000

GPIODATA    equ   0x000 ; GPIO Data       p416
GPIODIR     equ   0x400 ; GPIO Direction  p417 

GPIOIS      equ   0x404 ; GPIO Interrupt Sense        p418
GPIOIBE     equ   0x408 ; GPIO Interrupt Both Edges   p419
GPIOIEV     equ   0x40C ; GPIO Interrupt Event        p420

GPIOIM      equ 0x410   ; R/W 0x0000.0000 GPIO Interrupt Mask           p421
GPIORIS     equ 0x414   ; RO  0x0000.0000 GPIO Raw Interrupt Status     p422
GPIOMIS     equ 0x418   ; RO  0x0000.0000 GPIO Masked Interrupt Status  p423
GPIOICR     equ 0x41C   ; W1C 0x0000.0000 GPIO Interrupt Clear          p425

GPIOAFSEL   equ 0x420   ; R/W - GPIO Alternate Function Select          p426

GPIODR2R    equ 0x500   ; R/W 0x0000.00FF GPIO 2-mA Drive Select        p428
GPIODR4R    equ 0x504   ; R/W 0x0000.0000 GPIO 4-mA Drive Select        p429
GPIODR8R    equ 0x508   ; R/W 0x0000.0000 GPIO 8-mA Drive Select        p430

GPIOODR     equ 0x50C   ; R/W 0x0000.0000 GPIO Open Drain Select        p431
GPIOPUR     equ 0x510   ; R/W - GPIO Pull-Up Select                     p432
GPIOPDR     equ 0x514   ; R/W 0x0000.0000 GPIO Pull-Down Select         p434
GPIOSLR     equ 0x518   ; R/W 0x0000.0000 GPIO Slew Rate Control Select p436

GPIODEN     equ 0x51C   ; R/W - GPIO Digital Enable                     p437

GPIOLOCK    equ 0x520   ; R/W 0x0000.0001 GPIO Lock                     p439
GPIOCR      equ 0x524   ; - - GPIO Commit                               p440
GPIOAMSEL   equ 0x528   ; R/W 0x0000.0000 GPIO Analog Mode Select       p442
GPIOPCTL    equ 0x52C   ; R/W - GPIO Port Control                       p444

GPIOPeriphID4 equ  0xFD0  ; RO 0x0000.0000 GPIO Peripheral Identification 4 p446
GPIOPeriphID5 equ  0xFD4  ; RO 0x0000.0000 GPIO Peripheral Identification 5 p447
GPIOPeriphID6 equ  0xFD8  ; RO 0x0000.0000 GPIO Peripheral Identification 6 p448
GPIOPeriphID7 equ  0xFDC  ; RO 0x0000.0000 GPIO Peripheral Identification 7 p449
GPIOPeriphID0 equ  0xFE0  ; RO 0x0000.0061 GPIO Peripheral Identification 0 p450
GPIOPeriphID1 equ  0xFE4  ; RO 0x0000.0000 GPIO Peripheral Identification 1 p451
GPIOPeriphID2 equ  0xFE8  ; RO 0x0000.0018 GPIO Peripheral Identification 2 p452
GPIOPeriphID3 equ  0xFEC  ; RO 0x0000.0001 GPIO Peripheral Identification 3 p453

GPIOPCellID0 equ  0xFF0   ; RO 0x0000.000D GPIO PrimeCell Identification 0 p454
GPIOPCellID1 equ  0xFF4   ; RO 0x0000.00F0 GPIO PrimeCell Identification 1 p455
GPIOPCellID2 equ  0xFF8   ; RO 0x0000.0005 GPIO PrimeCell Identification 2 p456
GPIOPCellID3 equ  0xFFC   ; RO 0x0000.00B1 GPIO PrimeCell Identification 3 p457

PORT0	   EQU	0x01	;  
PORT1 	EQU	0x02	;
PORT2	   EQU	0x04	;
PORT3  	EQU	0x08	;
PORT4 	EQU	0x10	;
PORT5 	EQU	0x20	;
PORT6    EQU	0x40	;
PORT7	   EQU	0x80  ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; les constantes de dure 
; calculs apres mesure sur l'oscillo
;
D1s      equ   0x002FFFFF  ; en ralit 1.2s	
D5ms     equ   26600
D2ms     equ   10640
D150us   equ   800
D75us    equ   400
D50us    equ   266
D500ns   equ   3           ; en ralit 563ns

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; les GPIOS pour l'ecran LCD
; LCD connect sur le Port H + Port C comme suit: 
;    PH7..PH4 => DB7..DB4
;    PH3      => E
;    PH2      => R/W
;    PC4      => RS
;
DB7   equ   PORT7
DB6   equ   PORT6
DB5   equ   PORT5
DB4   equ   PORT4
E     equ   PORT3
RW    equ   PORT2
RS    equ   PORT4

	  	ENTRY

      export   init_LCD
init_LCD
     push {r0,r3,r6,lr}
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Init des registres GPIOs
;
; activation des horloges des ports C et H
;
      ldr r6, = SYSCTL_RCGC2 ; voir p412 Init et config des GPIOs
      mov r0, #0x00000084    ; Enable clock sur GPIOC + GPIOF p292
      str r0, [r6]           ;
		nop	                 ; tres tres important....merci Thierry (beaucoup temps perdu, cf petite note p414!)
		nop                    ; "There must be  a delay of 3 system clocks before any GPIO reg. access
		nop	                 ; pas necessaire en simu  ou en debbug step by step...@#@! :-(

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Port C (signal RS) direction
      ldr r6, = GPIO_PORTC_BASE + GPIODIR ;  Pins du port C en sortie
      ldr r0, [r6]
      add r0, #PORT4    ; singal RS en sortie
      str r0, [r6]
; Port C (RS) intensit	
;		ldr r6, = GPIO_PORTC_BASE + GPIODR2R	; Choix de l'intensit de sortie (2mA)
;     ldr r0, = PORT4 
;     str r0, [r6]
; Port C (RS) Enable
      ldr r6, = GPIO_PORTC_BASE + GPIODEN ;  Pins du port C  enable
      ldr r0, [r6]
      add r0, #PORT4
      str r0, [r6]	   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; port H direction  (signaux DB7-4, E et R/W)
      ldr r6, = GPIO_PORTH_BASE + GPIODIR ;  Pins du port H en sortie
      ldr r0, = (DB7 + DB6 + DB5 + DB4 + E + RW)
      str r0, [r6]
; port H intensit(DB7-4, E et R/W)	
;		ldr r6, = GPIO_PORTH_BASE + GPIODR2R	;; Choix de l'intensit de sortie (2mA)
;      ldr r0, = (DB7 + DB6 + DB5 + DB4 + E + RW)
;      str r0, [r6]
; port H enable (DB7-4, E et R/W)
      ldr r6, = GPIO_PORTH_BASE + GPIODEN ;  Pins du port H enable
      ldr r0, = PORT7 + PORT6 + PORT5 + PORT4 + PORT3 + PORT2
      str r0, [r6]
;
; on place touts les signaux a 0 en particulier RW et E
;
      ldr r6, = GPIO_PORTH_BASE + GPIODATA + ((DB7 + DB6 + DB5 + DB4 + E + RW) << 2)
      mov r3, #0
      str r3, [r6]
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  code principal du LCD
;
      bl initlcd           ; 
      pop {r0,r3,r6,pc}
      
;      ldr r0, =message1
;      bl putsLCD

;      bl RSir                 ; selectionne le registre d'instruction du LCD
;      ldr r1, = (0x80 + 0x40) ; pour placer le curseur au debut de la 2ieme ligne @0x40
;      bl WriteLCD
;      mov r1, #D50us          ; attendre >38us
;      bl wait

;     ldr r0, =message2
;      bl putsLCD

;loop b  loop                  

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sequence d'Initialisation du LCD
;

initlcd
      push {r1,r2,lr}
      ldr r1, = D1s	; attendre >4.1ms
      bl wait

      bl RSir        ; on selectionne le registre d'instruction du lcd 

      mov r2, #0x30  ; 1ier
      bl Write4bLCD
      mov r1, #D5ms	; attendre >4.1ms
      bl wait

      bl Write4bLCD  ; 2ieme
      mov r1, #D150us; attendre >100Uus
      bl wait

      bl Write4bLCD  ; 3ieme
      mov r1, #D150us;  attendre >100us
      bl wait

      mov r2, #0x20  ;
      bl Write4bLCD
      mov r1, #D150us;  attendre >100us
      bl wait

      mov r1,#0x28   ; 2 lignes + matrice de 8x5 points
      bl WriteLCD
      mov r1, #D50us;  attendre >38us
      bl wait

      mov r1,#0x0f   ; Display ON + Cursor ON + Blinking ON
      bl WriteLCD
      mov r1, #D50us;  attendre >38us
      bl wait

      ldr r1, = 0x01  ; clear display 
      bl WriteLCD
      mov r1, #D2ms;  attendre >1.52ms
      bl wait
      pop {r1,r2,pc}
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Procedure d'attente en fonction de r1

wait  push {lr}
wait_loop
      subs r1, #1
      bne wait_loop
      pop {pc}

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; RS <= 0 pour selectionner le registre d'instruction du lcd
;
RSir  push {r3,r6,lr}
      ldr r6, = GPIO_PORTC_BASE + GPIODATA + (RS << 2)
      mov r3, #0
      str r3, [r6]
      pop {r3,r6,pc}
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; RS <= 1 pour selectionner le registre de donnes du lcd
;
RSdr  push{r3,r6,lr}
      ldr r6, = GPIO_PORTC_BASE + GPIODATA + (RS << 2)
      mov r3, #RS
      str r3, [r6]
      pop {r3,r6,pc}

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ecriture des 4b de PF de r2 vers le registre IR ou DR du LCD.
; le signal RS ayant ete pralablement positionn pour choisr IR ou DR
;
Write4bLCD
      push {r2,r3,r6,lr}   ; sauvergarde du lr pour prevenir les appels imbriqus
      ; R/W <= 0 
      ; E   <= 0 dure du cycle Tc = > 1000ns
      ;
      ldr r6, = GPIO_PORTH_BASE + GPIODATA + ((RW + E) << 2)
      mov r3, #0
      str r3, [r6]
      nop   ; attendre Tsp1 = >60ns avant de positionner E a 1
      ;
      ; E <= 1 doit rester a 1 Tpw = >450ns
      ;
      ldr r6, = GPIO_PORTH_BASE + GPIODATA + (E << 2)
      mov r3, #E
      str r3, [r6]
      ;
      ; on positionne au plutot (Td= <360ns) les 
      ; 4b de PF de r2 sur les signaux de data
      ;
      ldr r6, = GPIO_PORTH_BASE + GPIODATA + ((DB7 + DB6 + DB5 + DB4) << 2)
      str r2,[r6]
      nop
      nop   ; pour atteindre Tpw = >450ns  
      ;
      ; E <= 0 
      ;
      ldr r6, = GPIO_PORTH_BASE + GPIODATA + (E << 2)
      mov r3, #0
      str r3, [r6]
      nop
      nop
      nop   ; pour attendre Tc = >1000ns (10000ns - Tpw = >550ns)
      nop   
      pop {r2,r3,r6,pc} ; restaure le lr
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ecriture de r1 vers le registre IR ou DR du LCD.
; le signal RS doit avoir ete pralablement positionn pour choisr IR ou DR
;
WriteLCD
      push {r1,r2,lr}         ; imbrication d'appel
      mov r2, r1        ; on place dans le PF de r2 les 4b de PF de r1
      bl Write4bLCD

      mov r2, r1, LSL#4 ; on place dans le PF de r2 les 4b de pf de r1
      bl Write4bLCD

      mov r1, #D50us
      bl wait
      pop {r1,r2,pc}          ; retour
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; putsLCD
; r0 = @ debut de chaines de caracteres termin par 0 pour la fin de chaine
;
         export puts_LCD
puts_LCD
         push  {r1,lr}        ; pour l'imbrication d'appel

         BL RSdr           ; on oublie pas de selectionner le reg de donnes

TantQue  ldrb  r1,[r0],#1  ; r1 <= le caractere courant puis +1 sur r0
         cmp   r1,#0       ; r1 = la fin de chaine ?
         beq   fin_putsLCD
         bl    WriteLCD    ; affiche le caractere courant
         b     TantQue
fin_putsLCD
         pop {r1,pc}    ; retour
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
message1  dcb "Dept I.T. sept13",0
message2  dcb "  TP EIG 2004   ",0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

      END 