//****************************************************************************
//
// chronos_drive.c - Controle l'EvalBot via UART bas sur Remote control of
// the EVALBOT using an eZ430-Chronos.
//
//****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
#include "sounds.h"
#include <string.h>
#include "driverlib/uart.h"
#include "driverlib/pwm.h"

//*****************************************************************************
//
// For debug purposes, define this label to keep the motors disabled at all
// times.
//
//*****************************************************************************
//#define KEEP_MOTORS_DISABLED

//*****************************************************************************
//
// Defines
//
//*****************************************************************************
#define CLEAR 1
#define HideCurs    "\033[?25l"
#define ClearScreen "\033[H\033[2J"

//*****************************************************************************
//
// Main application entry point.
//
//*****************************************************************************
int
main (void)
{
	unsigned long int j;
	int i,c;
    //
    // Set the system clock to run at 50MHz from the PLL
    //
    SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_16MHZ);
    //
    // Enable UART0.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);

    //
    // Initialize the UART 0 port serie sur USB
    //  

	UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), 115200,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));

    //
    // Initialize and Enable UART1 port serie sur pins
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART1);

	GPIOPinConfigure(GPIO_PB4_U1RX);
    GPIOPinConfigure(GPIO_PB5_U1TX);

    GPIOPinTypeUART(GPIO_PORTB_BASE, GPIO_PIN_4 | GPIO_PIN_5);

	UARTConfigSetExpClk(UART1_BASE, SysCtlClockGet(), 9600,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
        
		                 UART_CONFIG_PAR_NONE));
 
    //
    // Initialize the UART standard I/O.
    //
    UARTStdioInit(0); // pour rediriger es STDIO sur le UART O (port serie usb)
 
    //
    // Print a nice welcoming banner on the serial output.
    //

	UARTprintf(ClearScreen);  // efface la console 
    UARTprintf("EVALBOT PWM\n");
    UARTprintf("-----------\n");
	UARTprintf(HideCurs);	  // efface le curseur
    //
    // Initialize the LED display.
    //
    Display96x16x1Init(true);
    Display96x16x1Clear();
	Display96x16x1StringDrawCentered("ESIEE Info & Tel", 0, CLEAR);
	Display96x16x1StringDrawCentered("TEST PWM", 1, CLEAR);
	  
	//
	// Configure the PWM generator for count down mode with immediate updates
	// to the parameters.
	//
	SysCtlPeripheralEnable( SYSCTL_PERIPH_PWM0  ); // active le PWM_GEN_0
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOJ ); // active le port J

	GPIOPinConfigure( GPIO_PJ1_PWM1 );	           // PIN PWM1 sur PJ1
	GPIOPinTypePWM( GPIO_PORTJ_BASE, GPIO_PIN_1 ); // set GPIO J1 en PWM

	PWMGenConfigure(PWM_BASE, PWM_GEN_0,
	                PWM_GEN_MODE_DOWN | PWM_GEN_MODE_NO_SYNC);
	//
	// Set the period. For a 50 KHz frequency, the period = 1/50,000, or 20
	// microseconds. For a 20 MHz clock, this translates to 400 clock ticks.
	// Use this value to set the period.
	//
	PWMGenPeriodSet(PWM_BASE, PWM_GEN_0, 400);
	//
	// Set the pulse width of PWM1 for a 25% duty cycle.
	//
	PWMPulseWidthSet(PWM_BASE, PWM_OUT_1, 100);

	//
	// Start the timers in generator 0.
	//
	PWMGenEnable(PWM_BASE, PWM_GEN_0);
	//
	// Enable the outputs.
	//
	PWMOutputState(PWM_BASE, (PWM_OUT_1_BIT), true); // enable PWM1 sur PJ1
	i = 10;
	c = 5;
	for(;;) {
		PWMPulseWidthSet(PWM_BASE, PWM_OUT_1, i);
		for(j=0;j<200000;j++);
		if (i<10 || i>395) c = -c;
		i += c;
	}
}
