//****************************************************************************
//
// EvalBot : exmple d'utilisation de
// Oled
// UART0 ou UART1
// Sonar Ultra-son SRF05 avec GPIO (Irq) + TImer
//
//****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
#include "sounds.h"
#include <string.h>
#include "driverlib/uart.h"
#include "driverlib/timer.h"

//*****************************************************************************
//
// Defines
//
//*****************************************************************************
#define CLEAR 1
#define HideCurs    "\033[?25l"
#define ClearScreen "\033[H\033[2J"

static unsigned int p;
static unsigned char rising;


//*****************************************************************************
// procedure de traitement de L'it Echo du module SRF05
//*****************************************************************************
static void PortBIntHandler()
{
	GPIOPinIntClear( GPIO_PORTB_BASE, (GPIO_PIN_2) );  // raz 
	if (rising == 0) {  // front montant on lance le timer
		TimerLoadSet(TIMER0_BASE, TIMER_A, 0xffffff);
		TimerEnable(TIMER0_BASE, TIMER_A);
		rising = 1; // memo du front montant
	} else {	// front descendant lecture du timer et arret
		p = TimerValueGet(TIMER0_BASE, TIMER_A);
		TimerDisable(TIMER0_BASE, TIMER_A);
		rising = 0; // memo du front descendant 
	}
}
//*****************************************************************************
// MAIN  MAIN  MAIN  MAIN
//*****************************************************************************

int main (void)
{
	static unsigned long int i;
   unsigned int t;
   
   //
   // Set the system clock to run at 50MHz from the PLL
   //
   SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_16MHZ);
   
   //
   // Init UART0 port serie sur usb.
   //
   SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
   SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
   GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
  	UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), 115200,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));

   //
   // InitUART1 port serie sur GPIO vers le Xbee (ou Bluetooth)
   //
   SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
   SysCtlPeripheralEnable(SYSCTL_PERIPH_UART1);
   GPIOPinTypeUART(GPIO_PORTB_BASE, GPIO_PIN_4 | GPIO_PIN_5);
   GPIOPinConfigure(GPIO_PB4_U1RX);
   GPIOPinConfigure(GPIO_PB5_U1TX);
	UARTConfigSetExpClk(UART1_BASE, SysCtlClockGet(), 115200,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));
 
   //
   // Init l'UART des standard I/O.
   //
   UARTStdioInit(0); // pour rediriger les STDIO sur :
                     // 0 =  UART 0 (port serie USB)
                     // 1 =  UART 1 (port serie Xbee)
 
   //
   // Affiche un message sur la sortie serie standard
   //
	UARTprintf( ClearScreen );  // efface la console 
   UARTprintf( "EVALBOT INF201 SRF05\n" );
   UARTprintf( "--------------------\n" );
	UARTprintf( HideCurs );	  // efface le curseur

   //
   // Init du oLED EvalBot.
   //
   Display96x16x1Init(true);
   Display96x16x1Clear();
	Display96x16x1StringDrawCentered("Test SRF05", 0, CLEAR);
   //
   // Init des 2 LEDs a l'avant de l'EvalBot.
   //
   LEDsInit();

	
	//////////////////////////////////
	// INIT gpio pour le module SRF05
	//  entree Trigger <= PB3
   //  sortie Echo    => PB2
   //
	GPIOPinTypeGPIOOutput( GPIO_PORTB_BASE, GPIO_PIN_3 ); // PB3 en sortie Trigger SRF05
	GPIOPinTypeGPIOInput( GPIO_PORTB_BASE, GPIO_PIN_2 ); // PB2 en entre Echo    SRF05

	// PB3 en sortie standard + 2 mA 
	GPIOPadConfigSet( GPIO_PORTB_BASE, (GPIO_PIN_2 | GPIO_PIN_3), GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );

   // init PB2 Echo pour IT sur fronts montant et descandant
   p = 0;
 	GPIOPortIntRegister( GPIO_PORTB_BASE, PortBIntHandler );
	GPIOPinIntEnable( GPIO_PORTB_BASE, GPIO_PIN_2 );     
	GPIOIntTypeSet( GPIO_PORTB_BASE, GPIO_PIN_2, GPIO_BOTH_EDGES );
   GPIOPinIntEnable( GPIO_PORTB_BASE, GPIO_PIN_2 );
   GPIOPinWrite( GPIO_PORTB_BASE,(GPIO_PIN_3), 0x00); // trigger <= 0
   

	// SRF05 l'echo mesure entre 0,100ms et 25ms  = 30ms si aucun objet detect
	//
	SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER0);

	TimerConfigure(TIMER0_BASE, ( TIMER_CFG_SPLIT_PAIR | TIMER_CFG_A_ONE_SHOT ));

	TimerPrescaleSet( TIMER0_BASE, TIMER_A, 100);  // on divise l'horloge systeme 
	IntMasterEnable();

	for (;;) {
      
      //////////////////////////////////////
		// Pulse 15us mini vers SRF05
      rising = 0;
      GPIOPinWrite( GPIO_PORTB_BASE,(GPIO_PIN_3), 0xff); // trigger <= 1
      for (i=0;i<300;i++); // 15 us ?
      GPIOPinWrite( GPIO_PORTB_BASE,(GPIO_PIN_3), 0x00);  // trigger <= 0
      for (i=0;i<300;i++);
      
		UARTprintf("\33[4;5H          ");		UARTprintf("\33[4;5H %d\n", 65535-p );
      
		//Display96x16x1ClearLine(1);
      t = 65535 - p;
      
      Display96x16x1StringDrawCenteredPrintf(0,1,"%u", t );
      
      if      (t>600) Display96x16x1StringDrawCentered("################", 0, 0);
		else if (t>550) Display96x16x1StringDrawCentered("############### ", 0, 0);
		else if (t>500) Display96x16x1StringDrawCentered("##############  ", 0, 0);
		else if (t>450) Display96x16x1StringDrawCentered("#############   ", 0, 0);
		else if (t>400) Display96x16x1StringDrawCentered("############    ", 0, 0);
		else if (t>350) Display96x16x1StringDrawCentered("###########     ", 0, 0);
		else if (t>300) Display96x16x1StringDrawCentered("##########      ", 0, 0);
		else if (t>280) Display96x16x1StringDrawCentered("#########       ", 0, 0);
		else if (t>260) Display96x16x1StringDrawCentered("########        ", 0, 0);
		else if (t>240) Display96x16x1StringDrawCentered("#######         ", 0, 0);
		else if (t>220) Display96x16x1StringDrawCentered("######          ", 0, 0);
		else if (t>200) Display96x16x1StringDrawCentered("#####           ", 0, 0);
		else if (t>180) Display96x16x1StringDrawCentered("####            ", 0, 0);
		else if (t>160) Display96x16x1StringDrawCentered("###             ", 0, 0);
		else if (t>140) Display96x16x1StringDrawCentered("##              ", 0, 0);
		else if (t>120) Display96x16x1StringDrawCentered("#               ", 0, 0);
	}
}