//****************************************************************************
//
// EvalBot : exmple d'utilisation de
// ecran Oled
// UART0 ou UART1
//
//****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
#include "sounds.h"
#include <string.h>
#include "driverlib/uart.h"
#include "driverlib/timer.h"
//*****************************************************************************
//
// For debug purposes, define this label to keep the motors disabled at all
// times.
//
//*****************************************************************************
//#define KEEP_MOTORS_DISABLED

//*****************************************************************************
//
// Defines et variables globales
//
//*****************************************************************************
#define CLEAR 1
#define HideCurs    "\033[?25l"
#define ClearScreen "\033[H\033[2J"
static unsigned int p;
unsigned long i;
static unsigned char rising;
static unsigned int t,t1,t2;
//*****************************************************************************
//
// This function is called to Run/Stop the EVALBOT.
//
//*****************************************************************************
static void
EvalBotStop()
{
    //
    // Stop the motors.
    //
    MotorStop(LEFT_SIDE);
    MotorStop(RIGHT_SIDE);

}
static void
EvalBotRun()
{	
	MotorRun(LEFT_SIDE);
	MotorRun(RIGHT_SIDE);
}

void BumpDetection()
{
	tBoolean l, r;
	unsigned long int c;

	l = !BumpSensorGetStatus(LEFT_SIDE);
	r = !BumpSensorGetStatus(RIGHT_SIDE);

	if( l || r)	{
		EvalBotStop();
		if (l && r ) {
			MotorDir(LEFT_SIDE,  REVERSE);
			MotorDir(RIGHT_SIDE, REVERSE);
		}else{
			if (r) {
				MotorDir(LEFT_SIDE,  REVERSE);
				MotorDir(RIGHT_SIDE, FORWARD);
			}else{
				MotorDir(LEFT_SIDE,  FORWARD);
				MotorDir(RIGHT_SIDE, REVERSE);
			}
		}
		EvalBotRun();
		Display96x16x1StringDrawCentered("!! COLLISION !!", 1, CLEAR);
		UARTprintf("\33[6;0H");
		UARTprintf("Collision          ");		
		for(c=0;c<13000000;c++);
	}
}

//*****************************************************************************
// Ecriture LCD
// RS sur PC4
// R/W sur PH2
// E sur PH3
// DB7..DB4 sur PH7..PH4
//*****************************************************************************
void write_4b_LCD(unsigned char v)
{
   static unsigned char i;

   // On selectionne le Registre d'Instruction : RS <= 0
   GPIOPinWrite( GPIO_PORTC_BASE, GPIO_PIN_4, 0x00); 

   // Cycle d'ecritue des 4 bits de poids forts dans le registre d'Instruction du lcd  
   // R/W <= 0 + E <=0
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, ((v<<4) & 0xf0));
   // attendre Tsp1 = 60ns
   //for (i=0;i<6;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<2;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<1;i++);
}

//**************************

void writeIR_LCD( unsigned char v)
{
   static unsigned int i;

   // Cycle d'ecritue des 4 bits de poids forts dans le registre d'Instruction du lcd  
   // R/W <= 0 + E <=0
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, (v & 0xf0));
   // attendre Tsp1 = 60ns
   for (i=0;i<1;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<4;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<5;i++);

   // On selectionne le Registre d'Instruction : RS <= 0
   GPIOPinWrite( GPIO_PORTC_BASE, GPIO_PIN_4, 0x00); 

   // write lower 4bits of Instruction Register 
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, ((v << 4) & 0xf0 ));
   // attendre Tsp1 = 60ns
   for (i=0;i<1;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<4;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<5;i++);
}

//**************************
void writeDR_LCD( unsigned char v)
{
   static unsigned int i;

   // Cycle d'ecritue des 4 bits de poids forts dans le registre d'Instruction du lcd  
   // R/W <= 0 + E <=0
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, (v & 0xf0));
   // attendre Tsp1 = 60ns
   for (i=0;i<2;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<5;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<6;i++);

   // On selectionne le Registre d'Instruction : RS <= 1
   GPIOPinWrite( GPIO_PORTC_BASE, GPIO_PIN_4, 0x10); 

   // write lower 4bits of Instruction Register 
   GPIOPinWrite( GPIO_PORTH_BASE, 0xFC, ((v << 4) & 0xf0 ));
   // attendre Tsp1 = 60ns
   for (i=0;i<2;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, GPIO_PIN_3 ); // E <= 1
   // attendre Tpw = 450ns
   for (i=0;i<5;i++);
   GPIOPinWrite( GPIO_PORTH_BASE, GPIO_PIN_3, 0x00 ); // E <= 0
   // attendre Tc = 1000ns - Tpw = 550ns avant un prochain echange
   for (i=0;i<6;i++);

}

void init_LCD()
{
   unsigned long i;
   
   write_4b_LCD(0x3);
   for (i=0;i<60000;i++); // 4.8ms
   write_4b_LCD(0x3);
   for (i=0;i<2000;i++); // 160 us
   write_4b_LCD(0x3);
   for (i=0;i<2000;i++); // 160 us

   write_4b_LCD(0x2);     // interface  4bits
   for (i=0;i<2000;i++); // 2000 us

   writeIR_LCD(0x28); // 2 lignes + matrice de 8x5 points
   for (i=0;i<2000;i++); // 160 us

   writeIR_LCD(0x0f);    // Display OFF
   for (i=0;i<2000;i++); // 160 us

   writeIR_LCD(0x14);  // Display clear
   for (i=0;i<2000;i++); // 160 us
	
	 writeIR_LCD(0x0c);  // cusor off + blink off
   for (i=0;i<2000;i++); // 160 us
}
void puts_LCD( char *s)
{
   static unsigned long i;
   while (*s != 0 )  {
      writeDR_LCD( *s );
      s++;
      for(i=0;i<10000;i++);
   }
}

//*****************************************************************************
// procedure de traitement de L'it Echo du module SRF05
//*****************************************************************************
static void PortBIntHandler()
{
	GPIOPinIntClear( GPIO_PORTB_BASE, (GPIO_PIN_2) );  // raz 
//   UARTStdioInit(0); UARTprintf(" IT "); UARTStdioInit(1);
	if (rising == 0) {  // front montant on lance le timer
		TimerLoadSet(TIMER0_BASE, TIMER_A, 0xffffff);
		TimerEnable(TIMER0_BASE, TIMER_A);
		rising = 1; // memo du front montant
	} else {	// front descendant lecture du timer et arret
		p = TimerValueGet(TIMER0_BASE, TIMER_A); 
      t = 65535 - p;      
		TimerDisable(TIMER0_BASE, TIMER_A);
		rising = 0; // memo du front descendant
   }
}
//*****************************************************************************
// procedure de traitement de L'it des Bumpers 
//*****************************************************************************
static void PortEIntHandler()
{
   //static unsigned long i;
	GPIOPinIntClear( GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 );  // raz 
   EvalBotStop();
   BumpDetection();
   
   /*Display96x16x1StringDrawCentered("Choc detecte !", 1, CLEAR);
   MotorDir(LEFT_SIDE,  REVERSE);
	MotorDir(RIGHT_SIDE, REVERSE);
   MotorSpeed(LEFT_SIDE,  35 << 8); 
   MotorSpeed(RIGHT_SIDE, 35 << 8);
   EvalBotRun();
   {
      i = GPIOPinRead( GPIO_PORTE_BASE,(GPIO_PIN_0 | GPIO_PIN_1));
   }while (!i);
   EvalBotStop();
   */
}
 /////////////////////////////////////
 // Pulse 15us mini vers SRF05
 
void pulse()
{      
   unsigned long i;
   rising = 0;
   GPIOPinWrite( GPIO_PORTB_BASE,(GPIO_PIN_3), 0xff);
   for (i=0;i<300;i++);
   GPIOPinWrite( GPIO_PORTB_BASE,(GPIO_PIN_3), 0x00);
   for (i=0;i<100000;i++);
}
//*****************************************************************************
//
// Main application entry point.
//
//*****************************************************************************
int
main (void)
{
   static unsigned long int i, m, mo;
   unsigned int o, f;
   unsigned char s,c;
	
   ////////////////////////////////////////////////////////////////
   // Set the system clock to run at 50MHz from the PLL
   //
   SysCtlClockSet( SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN | SYSCTL_XTAL_16MHZ);
   ///////////////////////////////////////////////////////////////
   // Init UART0 port serie sur usb.
   //
   SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOA );
   SysCtlPeripheralEnable( SYSCTL_PERIPH_UART0 );
   GPIOPinTypeUART( GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
   UARTConfigSetExpClk( UART0_BASE, SysCtlClockGet(), 115200,
                        ( UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE | UART_CONFIG_PAR_NONE ) );
   ////////////////////////////////////////////////////////////////
   // InitUART1 port serie sur GPIO vers le Xbee (ou Bluetooth)
   //
   SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOB );
   SysCtlPeripheralEnable( SYSCTL_PERIPH_UART1 );
   GPIOPinTypeUART( GPIO_PORTB_BASE, GPIO_PIN_4 | GPIO_PIN_5 );
   GPIOPinConfigure( GPIO_PB4_U1RX );
   GPIOPinConfigure( GPIO_PB5_U1TX );
   UARTConfigSetExpClk( UART1_BASE, SysCtlClockGet(), 9600,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE | UART_CONFIG_PAR_NONE) );
   ///////////////////////////////////////////
   // Init l'UART des I/O standard.
   // redirige les STDIO sur :
   // 0 =  UART 0 (port serie USB)
   // 1 =  UART 1 (port serie Xbee/BlueTooth)
   UARTStdioInit(1);
										 
   /////////////////////////////////////////////
   // INIT gpio pour l'ecran LCD INF201
   // LCD sur le Por H + PortC
   //    DB7..DB4 <=> PH7..PH4
   //    E   <= PH3
   //    R/W <= PH2
   //    RS  <= PC4
   //
   SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOC );
   SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOH );
   // PH7..PH2 en sortie
	GPIOPinTypeGPIOOutput( GPIO_PORTH_BASE, 
                         GPIO_PIN_7 | GPIO_PIN_6 | GPIO_PIN_5 |
                          GPIO_PIN_4 | GPIO_PIN_3 | GPIO_PIN_2 ); 
	// PH7..PH2 en sortie standard + 2 mA 
	GPIOPadConfigSet( GPIO_PORTH_BASE,
                     GPIO_PIN_7 | GPIO_PIN_6 | GPIO_PIN_5 |
                      GPIO_PIN_4 | GPIO_PIN_3 | GPIO_PIN_2,
                     GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );
   // PC4 => RS lcd
   GPIOPinTypeGPIOOutput( GPIO_PORTC_BASE, (GPIO_PIN_4) ); 
	GPIOPadConfigSet( GPIO_PORTC_BASE, GPIO_PIN_4, GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );
   //////////////////////////////////////
   // Initialisation du LCD
   //init_LCD();
   //writeIR_LCD(0x01); // efface l'ecran
	//writeIR_LCD(0x80); // debut de premier ligne
   //puts_LCD( "  Salut a tous !"); 
   //writeIR_LCD(0xc3); // debut de deuxime ligne
   //puts_LCD( " coucou !");

   /////////////////////////////////////////////////////////////////////////////////////
	// INIT gpio pour le module SRF05
	//  entree Trigger <= PB3
   //  sortie Echo    => PB2
   //
	GPIOPinTypeGPIOOutput( GPIO_PORTB_BASE, (GPIO_PIN_3) ); // PB3 en sortie Trigger SRF05
	GPIOPinTypeGPIOInput( GPIO_PORTB_BASE, (GPIO_PIN_2) );  // PB2 en entre Echo    SRF05
	// PB3 en sortie standard + 2 mA 
   GPIOPadConfigSet( GPIO_PORTB_BASE, (GPIO_PIN_2 | GPIO_PIN_3), GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );
   // init PB2 Echo pour IT sur fronts montant et descendant
 	GPIOPortIntRegister( GPIO_PORTB_BASE, PortBIntHandler );
   GPIOPinIntEnable( GPIO_PORTB_BASE, GPIO_PIN_2 );     
   GPIOIntTypeSet( GPIO_PORTB_BASE, GPIO_PIN_2, GPIO_BOTH_EDGES );
   GPIOPinIntEnable( GPIO_PORTB_BASE, GPIO_PIN_2 );
   GPIOPinWrite( GPIO_PORTB_BASE,(GPIO_PIN_3), 0x00);
   
   //
   // SRF05 l'echo mesure entre 0,100ms et 25ms  = 30ms si aucun objet detect
   //
   SysCtlPeripheralEnable( SYSCTL_PERIPH_TIMER0 );
	TimerConfigure( TIMER0_BASE, ( TIMER_CFG_SPLIT_PAIR | TIMER_CFG_A_ONE_SHOT ));
	TimerPrescaleSet( TIMER0_BASE, TIMER_A, 100);  // on divise l'horloge systeme 
	IntMasterEnable();
   /////////////////////////////////////
   // Initialize the LED display.
   //
   Display96x16x1Init(true);
   Display96x16x1Clear();
	Display96x16x1StringDrawCentered("Poursuite", 0, CLEAR);
   //
   // Initialize the board LEDs.
   //
   LEDsInit();
   // 
   // Initialize the motors.
   //
   MotorsInit();
   //
   // Initialize the front bump sensors.
   //
   BumpSensorsInit();
 	GPIOPortIntRegister( GPIO_PORTE_BASE, PortEIntHandler );
   GPIOPinIntEnable( GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 );     
   GPIOIntTypeSet( GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1, GPIO_FALLING_EDGE );
   GPIOPinIntEnable( GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 );

   /* on engage une rotation pour detecter un objet proche */ 

MotorDir(LEFT_SIDE,  FORWARD);
   MotorDir(RIGHT_SIDE, REVERSE);
   MotorSpeed(LEFT_SIDE,  20 << 8); 
	MotorSpeed(RIGHT_SIDE, 20 << 8);
   EvalBotRun();
   
   do {
      pulse();
	} while( t>500);
   
   /* on a quelque chose */
   do { /* on continue pour detecter le bord de l'objet */
      pulse();
      m++; /* on mesure la largeur de l'objet */
   } while( t<500);
   
   mo = m/2;
   MotorDir(LEFT_SIDE, REVERSE);
   MotorDir(RIGHT_SIDE, FORWARD );            
   do{ /* on repart dans l'autre sens pour */
      pulse();
      m--;
   } while(m >= mo); /* s'arreter au milieu de l'objet */       
   EvalBotStop();

   MotorSpeed(LEFT_SIDE,  65 << 8); 
	MotorSpeed(RIGHT_SIDE, 65 << 8);

   
   
 /*  *****************
   t1 = t;
   for (;;) {
      pulse();
      t2 = t1;
      if (t<300) c = 1;
      else {
         if ((t>330) && (t<400)) c = 2;
         else {
            if (t < 400) c = 0; 
            else {
               MotorSpeed(LEFT_SIDE,  15 << 8); 
               //MotorSpeed(RIGHT_SIDE, 65 << 8);
               m = 0;
               do{ /* on repart dans l'autre sens pour */
                  pulse();
                  m++;
               } while( t>400 && m<300); /* s'arreter au milieu de l'objet */       
               MotorSpeed(LEFT_SIDE, 65 << 8);
               if (t>400) {
                  MotorDir(LEFT_SIDE,  FORWARD);
                  MotorDir(RIGHT_SIDE, REVERSE );
                  m=0;
                  do{ /* on repart dans l'autre sens pour */
                     pulse();
                     m++;
                  } while( t>400 && m<2000); /* s'arreter au milieu de l'objet */       
                  c = 0;
               } else c = 0;
            }
         }
      }
      if (s != c) {
         s = c;
         switch (c) {
            case 0:
               EvalBotStop();
            break;
            case 1:
               MotorDir(LEFT_SIDE,  FORWARD);
               MotorDir(RIGHT_SIDE, FORWARD);
               EvalBotRun();
            break;
            case 2:
               MotorDir(LEFT_SIDE,  REVERSE);
               MotorDir(RIGHT_SIDE, REVERSE);
               EvalBotRun();
            break;
         }
      }
   }
}
   */      
  /*          
            EvalBotStop();
         
      }
   o = 0;
   m = 0;
   f = 0;
   pulse();
   t1 = t2 = t;
   // objet est devant
   for (;;) {
      do {
         t1 = t2;
         //Display96x16x1StringDrawCenteredPrintf(1,0,"%u", t);
         pulse();
         for (i=0;i<100000;i++);
         t2 = t;
         t = t2-t1;
         EvalBotStop();
      } while ( (t>20) && (t<40));
      if (t<20) {
         MotorDir(LEFT_SIDE, FORWARD);
         MotorDir(RIGHT_SIDE, FORWARD );
      }else{
         MotorDir(LEFT_SIDE, REVERSE );
         MotorDir(RIGHT_SIDE, REVERSE );
      }
      EvalBotRun();
      
   }
    
   
   do {
      pulse();
     // Display96x16x1StringDrawCenteredPrintf(1,0,"%u", t);
      m++;
   } while( t<400);
   mo = m/2;
   MotorDir(LEFT_SIDE, REVERSE);
   MotorDir(RIGHT_SIDE, FORWARD );            
   do{
      pulse();
   //   Display96x16x1StringDrawCenteredPrintf(1,0,"%u", t);
      m--;
   } while(m >= mo);        
   EvalBotStop();
   
   do {
      pulse();
   } while (t<430);
 

   MotorDir( LEFT_SIDE, REVERSE );
   MotorDir( RIGHT_SIDE, REVERSE );      
   EvalBotRun();   
   if (t < 435) {
      MotorSpeed(LEFT_SIDE,  50 << 8); 
      MotorSpeed(RIGHT_SIDE, 50 << 8);
   } else {
      MotorSpeed(LEFT_SIDE,  20 << 8); 
      MotorSpeed(RIGHT_SIDE, 50 << 8); 
   }      
      m=0;
      do {
         pulse();
         m++;
      }while ( t>430 );
      EvalBotStop();
   
   for(;;);
}   


/*****************
   do {
      if (t>430) {
         MotorDir(LEFT_SIDE, FORWARD);
         MotorDir(RIGHT_SIDE, FORWARD );            
      } else {
         MotorDir(LEFT_SIDE,  REVERSE );
         MotorDir(RIGHT_SIDE, REVERSE );            
      }
   while (
      t1 = t2 = s = 0;
      
      t1 = t2;
      pulse();
      t2 = t;
      
      if ((t2 - t1) > 0 ) { if ( (s<99) && (s >0)) s += 1;}
      if ((t2 - t1) < 0 ) { if ( (s<99) && (s >0)) s -= 1;}
      if ((t2-t1) == 0) s = 0;
      s = 25;
      if (t > 430) 
      MotorSpeed(LEFT_SIDE,  s << 8); 
      MotorSpeed(RIGHT_SIDE, s << 8);
   }
}

         if (t >300) {
            MotorDir(LEFT_SIDE, REVERSE);
            MotorDir(RIGHT_SIDE, REVERSE); 
         } else {
            MotorDir(LEFT_SIDE, FORWARD);
            MotorDir(RIGHT_SIDE, FORWAR);
         }
         EvalBotRun();
         MotorSpeed(LEFT_SIDE,  0 << 8); 
	MotorSpeed(RIGHT_SIDE, 25 << 8);
   EvalBotRun();

   for (;;){
      do {
         pulse();
   //   Display96x16x1StringDrawCenteredPrintf(1,0,"%u", t);
      } while ( t<300);
      MotorSpeed(LEFT_SIDE,  25 << 8); 
      MotorSpeed(RIGHT_SIDE, 0 << 8);
   
      do {
         pulse();
         Display96x16x1StringDrawCenteredPrintf(1,0,"%u", t);
      }  while ( t<300);
      MotorSpeed(LEFT_SIDE,  0 << 8); 
      MotorSpeed(RIGHT_SIDE, 25 << 8);
   }
}
*/