//****************************************************************************
// EvalBot : exmple d'utilisation de
// ecran Oled + LCD
// UART0 + UART1 avec module radio (Xbee ou Bluetooth)
// SRF05
// moteurs
// bumpers
// switches
//****************************************************************************
#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
#include "sounds.h"
#include <string.h>
#include "driverlib/uart.h"
#include "driverlib/timer.h"
//*****************************************************************************
//
// Defines et variables globales
//
//*****************************************************************************
#define CLEAR 1
#define HideCurs    "\033[?25l"
#define ClearScreen "\033[H\033[2J"
static unsigned int p;
unsigned long i;
static unsigned char rising;
static long t,t1,t2,c,m;
//*****************************************************************************
//
// This function is called to Run/Stop the EVALBOT.
//
//*****************************************************************************
void EvalBotStop()
{
    //
    // Stop the motors.
    //
    MotorStop(LEFT_SIDE);
    MotorStop(RIGHT_SIDE);

}
//**************************************
void EvalBotRun()
{	
	MotorRun(LEFT_SIDE);
	MotorRun(RIGHT_SIDE);
}
//**************************************
void BumpDetection()
{
	tBoolean l, r;
	unsigned long int c;

	l = !BumpSensorGetStatus(LEFT_SIDE);
	r = !BumpSensorGetStatus(RIGHT_SIDE);

	if( l || r)	{
		EvalBotStop();
		if (l && r ) {
			MotorDir(LEFT_SIDE,  REVERSE);
			MotorDir(RIGHT_SIDE, REVERSE);
		}else{
			if (r) {
				MotorDir(LEFT_SIDE,  REVERSE);
				MotorDir(RIGHT_SIDE, FORWARD);
			}else{
				MotorDir(LEFT_SIDE,  FORWARD);
				MotorDir(RIGHT_SIDE, REVERSE);
			}
		}
		EvalBotRun();
		Display96x16x1StringDrawCentered("!! COLLISION !!", 1, CLEAR);
		UARTprintf("\33[6;0H");
		UARTprintf("Collision          ");		
		for(c=0;c<13000000;c++);
	}
}
//*****************************************************************************
// procedure de traitement de L'it des Bumpers 
//*****************************************************************************
static void PortEIntHandler()
{
   //static unsigned long i;
	GPIOPinIntClear( GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 );  // raz 
   EvalBotStop();
   BumpDetection();
   
   Display96x16x1StringDrawCentered("Choc detecte !", 1, CLEAR);
   MotorDir(LEFT_SIDE,  REVERSE);
	MotorDir(RIGHT_SIDE, REVERSE);
   MotorSpeed(LEFT_SIDE,  35 << 8); 
   MotorSpeed(RIGHT_SIDE, 35 << 8);
   EvalBotRun();
   {
      i = GPIOPinRead( GPIO_PORTE_BASE,(GPIO_PIN_0 | GPIO_PIN_1));
   }while (!i);
   EvalBotStop();
}
//*****************************************************************************
// procedure de traitement de L'it Echo du module SRF05
//*****************************************************************************
static void PortBIntHandler()
{
	GPIOPinIntClear( GPIO_PORTB_BASE, (GPIO_PIN_2) );  // raz 
//   UARTStdioInit(0); UARTprintf(" IT "); UARTStdioInit(1);
	if (rising == 0) {  // front montant on lance le timer
		TimerLoadSet(TIMER0_BASE, TIMER_A, 0xffffff);
		TimerEnable(TIMER0_BASE, TIMER_A);
		rising = 1; // memo du front montant
	} else {	// front descendant lecture du timer et arret
		p = TimerValueGet(TIMER0_BASE, TIMER_A); 
      t = 65535 - p;      
		TimerDisable(TIMER0_BASE, TIMER_A);
		rising = 0; // memo du front descendant
   }
}
/////////////////////////////////////
// Pulse 15us mini vers SRF05 
void pulse()
{      
   unsigned long i;
   rising = 0;
   GPIOPinWrite( GPIO_PORTB_BASE,(GPIO_PIN_3), 0xff);
   for (i=0;i<300;i++);
   GPIOPinWrite( GPIO_PORTB_BASE,(GPIO_PIN_3), 0x00);
   for (i=0;i<100000;i++);
}

//**************************************
void EvalBot( unsigned char c)
{
   switch (c) {
      case 0:
         EvalBotStop();
      break;
      case 1:
         MotorDir(LEFT_SIDE,  FORWARD);
         MotorDir(RIGHT_SIDE, FORWARD);
         EvalBotRun();
      break;
      case 2:
         MotorDir(LEFT_SIDE,  REVERSE);
         MotorDir(RIGHT_SIDE, REVERSE);
         EvalBotRun();
      break;
      case 3:
         MotorDir(LEFT_SIDE, FORWARD );
         MotorDir(RIGHT_SIDE, REVERSE);
         EvalBotRun();
      break;
      case 4:
         MotorDir(LEFT_SIDE, REVERSE );
         MotorDir(RIGHT_SIDE, FORWARD);
         EvalBotRun();
      break;
      case 5:
         MotorSpeed(LEFT_SIDE,  0 << 8); 
         MotorSpeed(RIGHT_SIDE, 40 << 8);
         //MotorDir(LEFT_SIDE,  REVERSE);
         MotorDir(RIGHT_SIDE, REVERSE);
         EvalBotRun();
      break;
      case 6:
         MotorSpeed(LEFT_SIDE,  40 << 8); 
         MotorSpeed(RIGHT_SIDE, 0 << 8);
         MotorDir(LEFT_SIDE, REVERSE );
         //MotorDir(RIGHT_SIDE, REVERSE);
         EvalBotRun();
      break;
    }
 }
//*************************************************************
extern void gpio_init_LCD(void);
extern void init_LCD(void);
extern void puts_LCD(char *s);
extern void curs1_LCD(void);
extern void curs2_LCD(void);
extern void cursoff_LCD(void);
//*****************************************************************************
// Main  Main Main  Main Main  Main Main  Main Main  Main Main  Main Main  Main
//*****************************************************************************
int main (void)
{
   ////////////////////////////////////////////////////////////////
   // Set the system clock to run at 50MHz from the PLL
   //
   SysCtlClockSet( SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN | SYSCTL_XTAL_16MHZ);

   ///////////////////////////////////////////////////////////////
   // Pport A : Init UART0 port serie sur usb.
   //
   SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOA );
   SysCtlPeripheralEnable( SYSCTL_PERIPH_UART0 );
   GPIOPinTypeUART( GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
   UARTConfigSetExpClk( UART0_BASE, SysCtlClockGet(), 115200, ( UART_CONFIG_WLEN_8 |
                        UART_CONFIG_STOP_ONE | UART_CONFIG_PAR_NONE ) );
   
   /////////////////////////////////////////////////////////////////////
   // Port B InitUART1 port serie sur GPIO vers le Xbee (ou Bluetooth)
   //
   SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOB );
   SysCtlPeripheralEnable( SYSCTL_PERIPH_UART1 );
   GPIOPinTypeUART( GPIO_PORTB_BASE, GPIO_PIN_4 | GPIO_PIN_5 );
   GPIOPinConfigure( GPIO_PB4_U1RX );
   GPIOPinConfigure( GPIO_PB5_U1TX );
   UARTConfigSetExpClk( UART1_BASE, SysCtlClockGet(), 9600, (UART_CONFIG_WLEN_8 |
                        UART_CONFIG_STOP_ONE | UART_CONFIG_PAR_NONE) );
                        
   ///////////////////////////////////////////
   // Init l'UART des I/O standard.
   // redirige les STDIO sur :
   // 0 =  UART 0 (port serie USB)
   // 1 =  UART 1 (port serie Xbee/BlueTooth)
   UARTStdioInit(1);
   
   //////////////////////////////////////////////
   // gpio pour l'ecran LCD 
   // LCD sur le Port H + Port C
   //    DB7..DB4 <=> PH7..PH4
   //    E   <= PH3
   //    R/W <= PH2
   //    RS  <= PC4
   //
//   SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOC );
//   SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOH );
   
   // PH7..PH2 en sortie
//    GPIOPinTypeGPIOOutput( GPIO_PORTH_BASE, 
//                        GPIO_PIN_7 | GPIO_PIN_6 | GPIO_PIN_5 |
//                          GPIO_PIN_4 | GPIO_PIN_3 | GPIO_PIN_2 ); 
	// PH7..PH2 en sortie standard + 2 mA 
//	GPIOPadConfigSet( GPIO_PORTH_BASE,
//                     GPIO_PIN_7 | GPIO_PIN_6 | GPIO_PIN_5 |
//                      GPIO_PIN_4 | GPIO_PIN_3 | GPIO_PIN_2,
//                     GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );
   // PC4 => RS lcd
//   GPIOPinTypeGPIOOutput( GPIO_PORTC_BASE, (GPIO_PIN_4) ); 
//	GPIOPadConfigSet( GPIO_PORTC_BASE, GPIO_PIN_4, GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );
   
   //////////////////////////////////////
   // Initialisation du LCD
   gpio_init_LCD();
   init_LCD();
   cursoff_LCD();
   puts_LCD( " Pour demarrer"); 
   curs2_LCD();    // debut de deuxime ligne
   puts_LCD( "pressez s1 + s2");

   
   /////////////////////////////////////
   // Init the LED display.
   //
   Display96x16x1Init(true);
   Display96x16x1Clear();
 
   /////////////////////////////////////////////////////////////////////////////////////
	// INIT gpio pour les switch utilisateur
	//  switch1 <= PD6
   //  switch2 <= PD7
   //
   SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
   GPIOPinTypeGPIOInput(GPIO_PORTD_BASE, GPIO_PIN_6 | GPIO_PIN_7);
   GPIOPadConfigSet(GPIO_PORTD_BASE, GPIO_PIN_6 | GPIO_PIN_7, GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD_WPU);
 
   Display96x16x1StringDrawCentered("c'est   s1 =>", 0, CLEAR);
   Display96x16x1StringDrawCentered("    ici s2 =>", 1, CLEAR);

   do {
      c = GPIOPinRead( GPIO_PORTD_BASE,(GPIO_PIN_6 | GPIO_PIN_7));
   }  while ( (c & 0xc0));
     
   /////////////////////////////////////////////////////////////////////////////////////
	// INIT gpio pour le module SRF05
	//  entree Trigger <= PB3
   //  sortie Echo    => PB2
   //
	GPIOPinTypeGPIOOutput( GPIO_PORTB_BASE, (GPIO_PIN_3) ); // PB3 en sortie Trigger SRF05
	GPIOPinTypeGPIOInput( GPIO_PORTB_BASE, (GPIO_PIN_2) );  // PB2 en entre Echo    SRF05
	// PB3 en sortie standard + 2 mA 
   GPIOPadConfigSet( GPIO_PORTB_BASE, (GPIO_PIN_2 | GPIO_PIN_3), GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );
   // init PB2 Echo pour IT sur fronts montant et descendant
 	GPIOPortIntRegister( GPIO_PORTB_BASE, PortBIntHandler );
   GPIOPinIntEnable( GPIO_PORTB_BASE, GPIO_PIN_2 );     
   GPIOIntTypeSet( GPIO_PORTB_BASE, GPIO_PIN_2, GPIO_BOTH_EDGES );
   GPIOPinIntEnable( GPIO_PORTB_BASE, GPIO_PIN_2 );
   GPIOPinWrite( GPIO_PORTB_BASE,(GPIO_PIN_3), 0x00);

   //
   // SRF05 l'echo mesure entre 0,100ms et 25ms  = 30ms si aucun objet detect
   //
   SysCtlPeripheralEnable( SYSCTL_PERIPH_TIMER0 );
	TimerConfigure( TIMER0_BASE, ( TIMER_CFG_SPLIT_PAIR | TIMER_CFG_A_ONE_SHOT ));
	TimerPrescaleSet( TIMER0_BASE, TIMER_A, 100);  // on divise l'horloge systeme 
	IntMasterEnable();

   //
   // Initialize the board LEDs.
   //
   LEDsInit();
   // 
   // Initialize the motors.
   //
   MotorsInit();
   //
   // Initialize the front bump sensors.
   //
   BumpSensorsInit();
 	GPIOPortIntRegister( GPIO_PORTE_BASE, PortEIntHandler );
   GPIOPinIntEnable( GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 );     
   GPIOIntTypeSet( GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1, GPIO_FALLING_EDGE );
   GPIOPinIntEnable( GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 );
   
   Display96x16x1StringDrawCentered("Dpt InfoTelecom", 0, CLEAR);
   curs1_LCD();  puts_LCD("JPO ESIEE 7 Dec");
   curs2_LCD();  puts_LCD("  pouruite...   ");
   Display96x16x1Clear();
   
ici:
   MotorSpeed(LEFT_SIDE,  30 << 8); 
	MotorSpeed(RIGHT_SIDE, 30 << 8);
   
//   Display96x16x1StringDrawCentered("recherche cible...", 1, CLEAR);

   EvalBot(3);
   
   do pulse(); while (t>700);
   t1 = t+10;
   MotorSpeed(LEFT_SIDE,  80 << 8); 
	MotorSpeed(RIGHT_SIDE, 80 << 8);

   do pulse(); while (t<=t1);
   EvalBot(0);
   EvalBot(4);
   do pulse(); while (t>700);
   //curs2_LCD(); puts_LCD("vu !");
   m=0;
   do {
         m++;
         pulse();
   } while (t<t1);
   
   EvalBot(3);
   m=m/2;
   do {
         m--;
         pulse();
   } while (m>=0);
   EvalBot(0);
      
   if (t>330) {
      EvalBot(2);
      do pulse(); while(t>330);
   } else {
      if (t<300) {
         EvalBot(1);
         do pulse(); while(t<300);
      }
   } 
   EvalBot(0);

   for(;;) {     
      pulse();
//      t2 = t - t1;
//      if (t2<0) t1 = t2 * -1; else t1 = t2;
      if (t>=300 && t<=330)
         EvalBot(0);
      else {
         if (t>330 && t<400)                  
            EvalBot(2);
         else {
            if (t<=330) EvalBot(1);
            else {  // >= 400
               EvalBot(3);
               for( c=0 ; c<25 ; c++) {
                  pulse();
                  if (t<400) break;
               }
               if (c>=25) {         
                  EvalBot(4);
                  for(c=0;c<100;c++) {
                     pulse();                  
                     if (t<400) break;             
                  }
                  if (c>=60) goto ici;
                  else for (c=0;c<20;c++) pulse();
               } else for (c=0;c<20;c++) pulse();
            }
         }
      }
   }
}
