//****************************************************************************
//
// chronos_drive.c - Controle l'EvalBot via UART bas sur Remote control of
// the EVALBOT using an eZ430-Chronos.
//
//****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
//#include "sounds.h"
#include <string.h>
#include "driverlib/uart.h"
#include "I2Clib.h"
#include "driverlib/timer.h"

//*****************************************************************************
//
// For debug purposes, define this label to keep the motors disabled at all
// times.
//
//*****************************************************************************
//#define KEEP_MOTORS_DISABLED

//*****************************************************************************
//
// Defines
//
//*****************************************************************************
#define CLEAR 1
#define HideCurs    "\033[?25l"
#define ClearScreen "\033[H\033[2J"

//*****************************************************************************
//
// This function is called to Run/Stop the EVALBOT.
//
//*****************************************************************************
static void
EvalBotStop()
{
    //
    // Stop the motors.
    //
    MotorStop(LEFT_SIDE);
    MotorStop(RIGHT_SIDE);

}
static void
EvalBotRun()
{	
	MotorRun(LEFT_SIDE);
	MotorRun(RIGHT_SIDE);
}

void BumpDetection()
{
	tBoolean l, r;
	unsigned long int c;

	l = !BumpSensorGetStatus(LEFT_SIDE);
	r = !BumpSensorGetStatus(RIGHT_SIDE);

	if( l || r)	{
		EvalBotStop();
		if (l && r ) {
			MotorDir(LEFT_SIDE,  REVERSE);
			MotorDir(RIGHT_SIDE, REVERSE);
		}else{
			if (r) {
				MotorDir(LEFT_SIDE,  REVERSE);
				MotorDir(RIGHT_SIDE, FORWARD);
			}else{
				MotorDir(LEFT_SIDE,  FORWARD);
				MotorDir(RIGHT_SIDE, REVERSE);
			}
		}
		EvalBotRun();
		Display96x16x1StringDrawCentered("!! COLLISION !!", 1, CLEAR);
		UARTprintf("\33[6;0H");
		UARTprintf("Collision          ");		
		for(c=0;c<13000000;c++);
	}
}
//*****************************************************************************
//
// Main application entry point.
//
//*****************************************************************************
int
main (void)
{
	unsigned long int i,j;
	char c;
	unsigned char v;
	char mess[5];
	
    //
    // Set the system clock to run at 50MHz from the PLL
    //
    SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_16MHZ);

    //
    // Perform any necessary SimpliciTI BSP initialization.
    //
 //   BSP_Init();

    //
    // Enable UART0.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);

    //
    // Initialize the UART standard I/O.
    //
    //UARTStdioInit(0);

//	UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), 9600,
//                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
//                         UART_CONFIG_PAR_NONE));

    //
    // Enable UART1.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART1);

	GPIOPinConfigure(GPIO_PB4_U1RX);
    GPIOPinConfigure(GPIO_PB5_U1TX);

    GPIOPinTypeUART(GPIO_PORTB_BASE, GPIO_PIN_4 | GPIO_PIN_5);

    //
    // Initialize the UART standard I/O.
    //
    UARTStdioInit(0);

	UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), 9600,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));
    //
    // Print a nice welcoming banner on the serial output.
    //

	UARTprintf(ClearScreen);  // efface la console 
	UARTprintf(HideCurs);	  // efface le curseur
    UARTprintf("EVALBOT test I2 + SRF02\n");
    UARTprintf("-----------------------\n");
    //
    // Initialize the LED display.
    //
    Display96x16x1Init(true);
    Display96x16x1Clear();
	Display96x16x1StringDrawCentered("TesT I2C + SRF02", 0, CLEAR);
    //
    // Initialize the board LEDs.
    //
    LEDsInit();
	I2CInit(GPIO_PB2_I2C0SCL);
	
	for (;;) {
		mess[0] = 0x00;
		mess[1] = 0x51;
		I2CWriteData(I2C0_MASTER_BASE, 0x70, mess, 2);

		for (i=0;i<800000;i++);

		mess[0] = 0x02;
		I2CWriteData(I2C0_MASTER_BASE, 0x70, mess, 1);
   		
		i = I2CReadData(I2C0_MASTER_BASE, 0x70, mess, 2);
		if (i == 2 ) { 
			j = mess[0]<<8;
			j |= mess[1];
			UARTprintf("\33[4;0Hdistance = %d cm ", j);
			if (j>50 )      Display96x16x1StringDrawCentered("################", 1, 0);
			else if ( j>45) Display96x16x1StringDrawCentered("############### ", 1, 0);
			else if ( j>40) Display96x16x1StringDrawCentered("##############  ", 1, 0);
			else if ( j>35) Display96x16x1StringDrawCentered("#############   ", 1, 0);
			else if ( j>32) Display96x16x1StringDrawCentered("############    ", 1, 0);
			else if ( j>30) Display96x16x1StringDrawCentered("###########     ", 1, 0);
			else if ( j>28) Display96x16x1StringDrawCentered("##########      ", 1, 0);
			else if ( j>26) Display96x16x1StringDrawCentered("#########       ", 1, 0);
			else if ( j>24) Display96x16x1StringDrawCentered("########        ", 1, 0);
			else if ( j>22) Display96x16x1StringDrawCentered("#######         ", 1, 0);
			else if ( j>20) Display96x16x1StringDrawCentered("######          ", 1, 0);
			else if ( j>18) Display96x16x1StringDrawCentered("#####           ", 1, 0);
			else if ( j>16) Display96x16x1StringDrawCentered("####            ", 1, 0);
			else if ( j>14) Display96x16x1StringDrawCentered("###             ", 1, 0);
			else if ( j>12) Display96x16x1StringDrawCentered("##              ", 1, 0);
			else if ( j>10) Display96x16x1StringDrawCentered("#               ", 1, 0);
		}
		for (i=0;i<800000;i++);
	}

	// The following example shows how to use the timer API to configure the timer as a half-width one
	// shot timer and a half-width edge capture counter.
	//
	// Configure TimerA as a half-width one-shot timer, and TimerB as a
	// half-width edge capture counter.
	//
	TimerConfigure(TIMER0_BASE, (TIMER_CFG_SPLIT_PAIR | TIMER_CFG_A_ONE_SHOT |
	TIMER_CFG_B_CAP_COUNT));
	//
	// Set the count time for the the one-shot timer (TimerA).
	//
	TimerLoadSet(TIMER0_BASE, TIMER_A, 3000);
	//
	// Configure the counter (TimerB) to count both edges.
	//
	TimerControlEvent(TIMER0_BASE, TIMER_B, TIMER_EVENT_BOTH_EDGES);
	//
	// Enable the timers.
	//
	TimerEnable(TIMER0_BASE, TIMER_BOTH);
 
}
