//****************************************************************************
//
// test_i2c_LSM303DLH.c - testsur EvalBot du magneto - accelero 
//
//   LSM303 Breakout ---------- EvalBot
//         Vin                   3.3v
//         GND                   GND
//         SDA                   PB3
//         SCL                   PB2
//****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
//#include "sounds.h"
#include <string.h>
#include "driverlib/uart.h"
#include "I2Clib.h"
#include "driverlib/timer.h"


//*****************************************************************************
//
// Defines
//
//*****************************************************************************
#define CLEAR 1
#define HideCurs    "\033[?25l"
#define ClearScreen "\033[H\033[2J"
#define byte unsigned char

#define SCALE 2  // accel full-scale, should be 2, 4, or 8
 
//
// des registres du LSM303
//
/* LSM303 Address definitions */
//#define LSM303_MAG  0x1E  // la pin SA0 est a la masse
#define LSM303_MAG  0x1E //0x3C
#define LSM303_ACC  0x19  // permet de distinguer un 2ieme accelero qui aurrait son SA0 au Vcc
//acceleration
#define CTRL_REG1_A 0x20
#define CTRL_REG2_A 0x21
#define CTRL_REG3_A 0x22
#define CTRL_REG4_A 0x23
#define CTRL_REG5_A 0x24
#define HP_FILTER_RESET_A 0x25
#define REFERENCE_A 0x26
#define STATUS_REG_A 0x27
#define OUT_X_L_A 0x28
#define OUT_X_H_A 0x29
#define OUT_Y_L_A 0x2A
#define OUT_Y_H_A 0x2B
#define OUT_Z_L_A 0x2C
#define OUT_Z_H_A 0x2D
#define INT1_CFG_A 0x30
#define INT1_SOURCE_A 0x31
#define INT1_THS_A 0x32
#define INT1_DURATION_A 0x33
// magnetude
#define CRA_REG_M 0x00
#define CRB_REG_M 0x01
#define MR_REG_M 0x02
#define OUT_X_H_M 0x03
#define OUT_X_L_M 0x04
#define OUT_Y_H_M 0x05
#define OUT_Y_L_M 0x06
#define OUT_Z_H_M 0x07
#define OUT_Z_L_M 0x08
#define SR_REG_M 0x09
#define IRA_REG_M 0x0A
#define IRB_REG_M 0x0B
#define IRC_REG_M 0x0C
//*****************************************************************************
//
// Variables globales
//
//*****************************************************************************

int accel[3];	  		// pour stocker les valeurs brutes de l'acceleration des 3 axes
int mag[3];  				// valeurs du champs magnetiques des 3 axes
float realAccel[3];	// pour calculer l'acceleration
char mess[10];

//*****************************************************************************
//
// fonctions LSM303
//
//*****************************************************************************
// Transfere I2C quand le maitre recoit 1 octet de donne de l'esclave
// Master ST SAD + W     SUB     SR SAD + R         NMAK SP
// Slave             SAK     SAK           SAK DATA
// le maitre commance par envoyer le start + adresse de l'ACC ou du MAG + W
// l'esclave retourne le AK
// le maitre envoie la sub adresse de registre de l'ACC ou du MAG
// L'escalave le AK
// le maitre envoie l'adresse de L'ACC ou du MAG + R
// l'esclave retourne le AK  puis les datas...
//
byte LSM303_read(byte address, byte qtt )
{
  byte dest, nb ;
  
  if (address >= 0x20)
    dest = LSM303_ACC+1;
  else
    dest = LSM303_MAG+1;
  
  mess[0] = address;
  
	I2CWriteData(I2C0_MASTER_BASE, dest, mess, 1);
	nb = I2CReadData(I2C0_MASTER_BASE, dest, mess, qtt);
	if (qtt > 1)
		return nb;
	else
		return  mess[0];
}
//=================================================
void LSM303_write(byte data, byte address)
{
  byte dest;
  if (address >= 0x20)
    dest = LSM303_ACC;
  else
    dest = LSM303_MAG;

	mess[0] = address;
	mess[1] = data;
	I2CWriteData(I2C0_MASTER_BASE, dest, mess, 2);
}
//=================================================
void initLSM303(int fs)
{
	//enable accelerometer
	mess[0] = CTRL_REG1_A;
	mess[1] = 0x27; // normal power mode, 50 Hz data rate, all axes enabled
	I2CWriteData(I2C0_MASTER_BASE, LSM303_ACC, mess, 2);  // write acc
//  if ((fs==8)||(fs==4))
//    LSM303_write((0x00 | (fs-fs/2-1)<<4), CTRL_REG4_A);  // set full-scale
//  else
//    LSM303_write(0x00, CTRL_REG4_A);
	
	//enable magnetometer
//  LSM303_write(0x02, CRA_REG_M);  // 0x14 = mag 30Hz output rate

	mess[0] =  MR_REG_M;
	mess[1] = 0x00;	// continuous conversion mode
  I2CWriteData(I2C0_MASTER_BASE, LSM303_MAG, mess, 2); 	// write mag
	
	return;
	}
//=================================================
void getLSM303_mag(int *rawValues)
{
	
	mess[0] = OUT_X_H_M; //0x03;
	I2CWriteData(I2C0_MASTER_BASE, LSM303_MAG, mess, 1);
  I2CReadData(I2C0_MASTER_BASE, LSM303_MAG, mess, 6);
	rawValues[0] = ((mess[0] << 8) | mess[1]);
	rawValues[1] = ((mess[2] << 8) | mess[3]);
	rawValues[2] = ((mess[4] << 8) | mess[5]);
}

void getLSM303_acc(int *rawValues)
{
	
	mess[0] = OUT_X_H_M; //0x03;
	I2CWriteData(I2C0_MASTER_BASE, LSM303_ACC, mess, 1);
  I2CReadData(I2C0_MASTER_BASE, LSM303_ACC, mess, 6);
	rawValues[0] = ((mess[0] << 8) | mess[1]);
	rawValues[1] = ((mess[2] << 8) | mess[3]);
	rawValues[2] = ((mess[4] << 8) | mess[5]);
}
//*****************************************************************************
//
// Main application entry point.
//
//*****************************************************************************
int main (void)
{
	unsigned long int i;
	
	//
  // Set the system clock to run at 50MHz from the PLL
  //
  SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_16MHZ);
  //
  // Enable UART0.
  //
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);	// active le port A
  SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);	// active l'uart 0
  GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);	// PA0 + PA1 mode UART

  //
  // Enable UART1.
  //
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB); 	// active le port B
  SysCtlPeripheralEnable(SYSCTL_PERIPH_UART1);	// active l'UART 1

	GPIOPinConfigure(GPIO_PB4_U1RX);	// PB4 pour la reception UART1
  GPIOPinConfigure(GPIO_PB5_U1TX);	// PB5 pour la transission UART1
  GPIOPinTypeUART(GPIO_PORTB_BASE, GPIO_PIN_4 | GPIO_PIN_5);	//PB4 + PB5  mode UART

  //
  // Initialize the UART standard I/O.
  //
  UARTStdioInit(0);	// redirige les UARTStdio vers l'uart 0

	UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), 9600,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));
  //
  // Print a nice welcoming banner on the serial output.
  //

	UARTprintf(ClearScreen);  // efface la console 
	UARTprintf(HideCurs);	  // efface le curseur
  UARTprintf("EVALBOT test I2 + LSM303\n");
  UARTprintf("------------------------\n");
  //
  // Initialize the LED display.
  //
  Display96x16x1Init(true);
  Display96x16x1Clear();
	Display96x16x1StringDrawCentered("TesT I2C+LSM303", 0, CLEAR);
		
	I2CInit(GPIO_PB2_I2C0SCL);
//	for(;;) {
		initLSM303(SCALE);
		for (i=0;i<10000;i++);
//	}
	
	for (;;) {
		while(!(LSM303_read(SR_REG_M, 1) & 0x01));  // wait for the magnetometer readings to be ready
		getLSM303_mag(mag);  	// get the magnetometer values, store them in mag
		getLSM303_acc(accel);  	// get the magnetometer values, store them in mag

//		UARTprintf("\33[4;0H                           ");
		UARTprintf("%x %x x=%x ### x=%x y=%x z=%x\n", mag[0], mag[1], mag[2], accel[3], accel[4], accel[5]);
		for (i=0;i<500000;i++);
	} 
}
