//****************************************************************************
//
// chronos_drive.c - Controle l'EvalBot via UART bas sur Remote control of
// the EVALBOT using an eZ430-Chronos.
//
//****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
//#include "sounds.h"
#include <string.h>
#include "driverlib/uart.h"
#include "I2Clib.h"
#include "driverlib/timer.h"
#include "driverlib/gpio.h"

//*****************************************************************************
//
// For debug purposes, define this label to keep the motors disabled at all
// times.
//
//*****************************************************************************
//#define KEEP_MOTORS_DISABLED

//*****************************************************************************
//
// Defines
//
//*****************************************************************************
#define CLEAR 1
#define HideCurs    "\033[?25l"
#define ClearScreen "\033[H\033[2J"

static int pSTATUS = 0;
unsigned long p1,p2,t;
unsigned long int i,j;


void PortCIntHandler(void)
{
    //
    // Enter an infinite loop.
    //
	GPIOPinIntClear(GPIO_PORTC_BASE, GPIO_PIN_4);

    switch (pSTATUS){
		case 0: 
	//		TimerEnable(TIMER1_BASE, TIMER_A);
	//		UARTprintf(" A ");
			p1 = SysTickValueGet();
			GPIOIntTypeSet(GPIO_PORTC_BASE, GPIO_PIN_4, GPIO_FALLING_EDGE);
			pSTATUS = 1;
    	break;
		case 1: 
	//		UARTprintf(" B ");
	//		t = TimerValueGet(TIMER1_BASE, TIMER_A); 
			p2 = SysTickValueGet();
		//	TimerDisable(TIMER1_BASE, TIMER_A);
			GPIOIntTypeSet(GPIO_PORTC_BASE, GPIO_PIN_4, GPIO_RISING_EDGE);
			pSTATUS = 2;
		break;
	}
}
//*****************************************************************************
//
// Main application entry point.
//
//*****************************************************************************

int
main (void)
{
    //
    // Set the system clock to run at 50MHz from the PLL
    //
    SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN | SYSCTL_XTAL_16MHZ);

    //
    // Enable UART0.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);

    //
    // Initialize the UART standard I/O.
    //
    //UARTStdioInit(0);

//	UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), 9600,
//                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
//                         UART_CONFIG_PAR_NONE));

    //
    // Enable UART1.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART1);

	GPIOPinConfigure(GPIO_PB4_U1RX);
    GPIOPinConfigure(GPIO_PB5_U1TX);

    GPIOPinTypeUART(GPIO_PORTB_BASE, GPIO_PIN_4 | GPIO_PIN_5);

    //
    // Initialize the UART standard I/O.
    //
    UARTStdioInit(0);

	UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), 9600,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));
    //
    // Print a nice welcoming banner on the serial output.
    //

	UARTprintf(ClearScreen);  // efface la console 
	UARTprintf(HideCurs);	  // efface le curseur
    UARTprintf("EVALBOT test TIMER + SRF05\n");
    UARTprintf("--------------------------\n");
    //
    // Initialize the LED display.
    //
    Display96x16x1Init(true);
    Display96x16x1Clear();
	Display96x16x1StringDrawCentered("TIMER + SRF05", 0, CLEAR);
	LEDsInit();

	//
	// Configure the device to run at 20 MHz from the PLL using a 4 MHz crystal
	// as the input.
	//
	// deja fait avant : SysCtlClockSet(SYSCTL_SYSDIV_10 | SYSCTL_USE_PLL | SYSCTL_XTAL_4MHZ | SYSCTL_OSC_MAIN);
	//
	// Enable the GPIO blocks and the SSI.
	//
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOH);

	//////////////////////////////////////////////////
	// Initialize the GPIO PC pin configuration
	// Ctrl du module SRF05
	// PC5 en sortie pour le Trig Pulse 10us Mini
	// PC4 pour mesurer la longueur de l'echo entre 100us et 25 ms
	//
	
//	GPIOPinTypeGPIOInput(GPIO_PORTC_BASE, GPIO_PIN_4); // PC4 
//	GPIOPinConfigure(GPIO_PC4_EPI0S2);

	GPIOPinTypeGPIOOutput(GPIO_PORTH_BASE, (GPIO_PIN_2) ); // PC4 en sortie 

//	GPIOIntTypeSet(GPIO_PORTC_BASE, GPIO_PIN_4, GPIO_RISING_EDGE);
	GPIOPadConfigSet(GPIO_PORTH_BASE, GPIO_PIN_2,
	                 GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD);

//	GPIOPortIntRegister(GPIO_PORTC_BASE, PortCIntHandler);

//	GPIOPinIntEnable(GPIO_PORTC_BASE, GPIO_PIN_4 );

	////////////////////////////////////////////////
	// Set pin 5 as output, SW controlled.
	//

//	GPIOPinTypeGPIOOutput(GPIO_PORTC_BASE, GPIO_PIN_4); // PC4 en sortie

//	SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER1);
//    TimerConfigure(TIMER1_BASE, TIMER_CFG_32_BIT_OS_UP);
//    TimerLoadSet(TIMER1_BASE, TIMER_A, 16000000);
    // Set the Timer0B load value to 1ms.
 //   TimerLoadSet(TIMER1_BASE, TIMER_A, SysCtlClockGet()/500);

	//
	// Configure and enable the SysTick counter.
	//
//	SysTickPeriodSet(1000);
//    SysTickPeriodSet(SysCtlClockGet()/10000);
//	SysTickEnable();
 
	pSTATUS = 2;
	
//	IntMasterEnable();

	 for (;;) {
	 	//if (pSTATUS == 2 )
		  {
			//pSTATUS = 0;
			//UARTprintf(" %x - %x = %x \n", p2, p1 , (p2 -p1));
			GPIOPinWrite(GPIO_PORTH_BASE, (GPIO_PIN_2), 0xff);
			for (i=0;i<10000;i++);
			GPIOPinWrite(GPIO_PORTH_BASE, (GPIO_PIN_2), 0x00);
			for (i=0;i<100000;i++);
		}
		//LED_On(BOTH_LEDS);
		//for (i=0;i<1000000;i++);
		//LED_Off(BOTH_LEDS);
		//for (i=0;i<1000000;i++);
	}
}
   
