//****************************************************************************
// EvalBot : exemple d'utilisation :
//  + ecran Oled
//  + UART0 (port serie via l'USB)
//  + UART1 (port serie via module Xbee ou BlueTooth)
//  + moteurs
//  + bumpers
//	+ leds
//  => lire de readme.txt fenetre Project
//****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/debug.h"
#include "driverlib/interrupt.h"
#include "driverlib/udma.h"
#include "drivers/display96x16x1.h"
#include "drivers/motor.h"
#include "drivers/io.h"
#include "drivers/sensors.h"
#include "drivers/dac.h"
#include "drivers/sound.h"
#include "drivers/wav.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "utils/scheduler.h"
#include "simplicitilib.h"
#include "sounds.h"
#include <string.h>
#include "driverlib/uart.h"
//*****************************************************************************
//
// Defines
//
//*****************************************************************************
#define CLEAR 1
#define HideCurs    "\033[?25l"
#define ClearScreen "\033[H\033[2J"
//*****************************************************************************
//
// Fonctions Run/Stop des moteurs.
//
//*****************************************************************************
static void EvalBotStop()
{
    //
    // Stop the motors.
    //
    MotorStop(LEFT_SIDE);
    MotorStop(RIGHT_SIDE);
}
static void EvalBotRun()
{	
	MotorRun(LEFT_SIDE);
	MotorRun(RIGHT_SIDE);
}
//*****************************************************************************
// BumpDetection
//*****************************************************************************
void BumpDetection()
{
	tBoolean l, r;
	unsigned long int c;

	l = !BumpSensorGetStatus(LEFT_SIDE);
	r = !BumpSensorGetStatus(RIGHT_SIDE);

	if( l || r)	{
		EvalBotStop();
		if (l && r ) {
			MotorDir(LEFT_SIDE,  REVERSE);
			MotorDir(RIGHT_SIDE, REVERSE);
		}else{
			if (r) {
				MotorDir(LEFT_SIDE,  REVERSE);
				MotorDir(RIGHT_SIDE, FORWARD);
			}else{
				MotorDir(LEFT_SIDE,  FORWARD);
				MotorDir(RIGHT_SIDE, REVERSE);
			}
		}
		EvalBotRun();
		Display96x16x1StringDrawCentered("!! COLLISION !!", 1, CLEAR);
		UARTprintf("\33[6;0H");
		UARTprintf("Collision          ");		
		for(c=0;c<13000000;c++);
	}
}
//*****************************************************************************
//
// Main application entry point.
//
//*****************************************************************************
int
main (void)
{
	unsigned long int i,j;
	char c;
	unsigned char v;
	
  //
  // Horloge systeme a 50MHz depuis la PLL
  //
  SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                 SYSCTL_XTAL_16MHZ);
	//
  // Init UART0 port serie 0 est connect vers l'usb hte (le PC).
  //
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA); // on active l'hologe du port A
  SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0); // on active l'hologe de l'uart 0
  GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1); // PA0 et PA1 en mode UART
  GPIOPinConfigure(GPIO_PA0_U0RX); // PA0 pour recevoir
  GPIOPinConfigure(GPIO_PA1_U0TX); // PA1 pour transmettre
  // UART0 : vitesse 115200 + 8 bits + pas de parit + 1 stop
  UARTConfigSetExpClk( UART0_BASE, SysCtlClockGet(),
                        115200, (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE | UART_CONFIG_PAR_NONE) );
  //
  // Init UART1 port serie 1 sur GPIO a connceter vers un Xbee, un Bluetooth ou autres....
  //
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB); // on active l'hologe du port B
  SysCtlPeripheralEnable(SYSCTL_PERIPH_UART1); // on active l'hologe de l'uart 1
  GPIOPinTypeUART(GPIO_PORTB_BASE, GPIO_PIN_4 | GPIO_PIN_5); // PB4 et PB5 en mode UART
  GPIOPinConfigure(GPIO_PB4_U1RX); // PB4 pour recevoir
  GPIOPinConfigure(GPIO_PB5_U1TX); // PB5 pour transmettre
  // UART1 : vitesse 115200 + 8 bits + pas de parit + 1 stop	 
	UARTConfigSetExpClk( UART1_BASE, SysCtlClockGet(),
											 115200, (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |UART_CONFIG_PAR_NONE));
  //
  // On redirige les standard I/O sur :
	// 0 vers UART 0 (port serie USB hte)
  // 1 vers UART 1 (port serie Xbee,...)
  //
  UARTStdioInit(1);                    
  //
  // Print a nice welcoming banner on the serial output.
  //
	UARTprintf(ClearScreen);  // efface la sortie standard
	UARTprintf(HideCurs);	   // fait disparaitre e curseur
  UARTprintf("EVALBOT test UART\n");
  UARTprintf("-----------------\n");
	UARTprintf("Reglage de la vitesse (PWM) avec les touches + et -\n");
	UARTprintf("deplacement avec les 4 touches fleches");
  //
  // Init Oled
  //
  Display96x16x1Init(true);
  Display96x16x1Clear();
	Display96x16x1StringDrawCentered("ESIEE IT/Llorens", 0, CLEAR);
	Display96x16x1StringDrawCentered("TEST UART", 1, CLEAR);
  //
  // autres INITS /
  //
  LEDsInit();
  MotorsInit();
	v = 30; // vitesse a 30%
	BumpSensorsInit();

	for (;;) {
		Display96x16x1StringDrawCentered(" ARRET !", 1, CLEAR);
		UARTprintf("\33[6;0H");
		UARTprintf("ARRET                  ");
		do {
			c = UARTgetc();
			switch (c) {
				case '+': {
					if (v < 99) v++; break;
				}
				case '-': {
					if (v > 10) v--; break;
				}
				default: break;
			}
			UARTprintf("\33[5;0H");
			UARTprintf("Vitesse a %d%%",v);
			MotorSpeed(LEFT_SIDE,  v << 8); 
			MotorSpeed(RIGHT_SIDE, v << 8);
		} while ( c != 27); // attendre la sequence ESC 
		c = UARTgetc(); // on devrait attendre le code 91
		c = UARTgetc(); 
		//	 UARTprintf("c = %d\n",c);
		
		switch (c) {
			case 65: { // FORWARD
				Display96x16x1StringDrawCentered("en AVANT !", 1, CLEAR);
				UARTprintf("\33[6;0H");
				UARTprintf("en AVANT !           ");
				MotorDir(LEFT_SIDE,  FORWARD);
				MotorDir(RIGHT_SIDE, FORWARD);
				break;
			}
			case 68: { // LEFT
				Display96x16x1StringDrawCentered("rotation gauche", 1, CLEAR);
				UARTprintf("\33[6;0H");
				UARTprintf("rotation a gauche    ");
				MotorDir(LEFT_SIDE,  REVERSE);
				MotorDir(RIGHT_SIDE, FORWARD);			     
				break;
			}
			case 67: { // RIGHT
				Display96x16x1StringDrawCentered("rotation droite", 1, CLEAR);
				UARTprintf("\33[6;0H");
				UARTprintf("rotation a droite    ");
				MotorDir(LEFT_SIDE,  FORWARD);
				MotorDir(RIGHT_SIDE, REVERSE);			     
				break;
			}
			case 66: { // REVERSE
				Display96x16x1StringDrawCentered("en ARRIERE !", 1, CLEAR);
				UARTprintf("\33[6;0H");
				UARTprintf("en ARRIERE !       ");
				MotorDir(LEFT_SIDE,  REVERSE);
				MotorDir(RIGHT_SIDE, REVERSE);
				break;
			}
			default:
				// pour tous autre carac on execute la precedente commande
			break;
		}

		EvalBotRun();
		// on fait clignoter les Leds pendant moteurs Run
		// Bumper en mode scrutation
		for(j=0;j<6;j++) {
			BumpDetection(); // test l'etat des Bumpers
			LED_On(LED_1);
			for (i=0;i<400000;i++);
			BumpDetection(); // test l'etat des Bumpers
			LED_Off(LED_1); LED_On(LED_2);
			for (i=0;i<400000;i++);
			LED_Off(LED_2); // test l'etat des Bumpers
		}
		EvalBotStop();
	}				
}
